
# Reciprocity and behavioral heterogeneity govern the stability of social networks 
# Roslyn Dakin and T. Brandt Ryder

# This script analyzes the correspondence between lek structure and natural cluster structure in the annual networks and generates Sankey plots
# Extended Data Figure 1

library(dplyr)
library(igraph)
library(googleVis)

# Get social interaction data:
year3 <- read.csv('filtered_interactions_nodups_times_durations_052418.csv', stringsAsFactors=F)[,c('date','nTime','Color1','Color2','Lek','studyyear','DL','sitefix')]; dim(year3)
year12 <- read.csv('filtered_interactions_nodups_times_durations_011018.csv', stringsAsFactors=F)[,c('date','nTime','Color1','Color2','Lek','studyyear','DL','sitefix')]; dim(year12)
ally <- rbind(year12, year3); head(ally); dim(ally) # Duplicate and reverse individual 1 & individual 2 to account for all interactions:
allyX <- ally
allyX$Color1 <- ally$Color2; allyX$Color2 <- ally$Color1
dim(ally); dim(allyX)
ally <- rbind(ally, allyX)
ally$Color1[ally$Color1=='OX-'] <- 'OX' # Correct ID error
ally$Color2[ally$Color2=='OX-'] <- 'OX'
ally$sitefix[ally$sitefix=='OX-'] <- 'OX'
all(levels(factor(ally$Color1))==levels(factor(ally$Color2))); dim(ally) # 73,770

# Cluster analysis on each of the three annual networks:
yr <- '15-16'
data1 <- subset(ally, studyyear==yr)[,c('Color1','Color2','sitefix','date','Lek')] ; dim(data1); length(table(data1$Lek))
g1a <- unique(data1[,c('Color1','Color2')])
g1a <- subset(g1a, Color1<Color2)
temp <- summarize(group_by(data1, Color1, Color2), n=n())
g1a$raw.weight <- temp$n[match(paste(g1a$Color1, g1a$Color2),  paste(temp$Color1, temp$Color2))]
g1w <- graph.data.frame(data.frame(Color1=g1a$Color1, Color2=g1a$Color2, weight=g1a$raw.weight), directed=F) # Network
head(data1);dim(data1)
(y1mod <- cluster_walktrap(g1w, steps=10, weights=E(g1w)$weight))
modularity(y1mod) # 0.70

yr <- '16-17'
data2 <- subset(ally, studyyear==yr)[,c('Color1','Color2','sitefix','date','Lek')]; dim(data2); length(table(data2$Lek))
g2a <- unique(data2[,c('Color1','Color2')])
g2a <- subset(g2a, Color1<Color2)
temp <- summarize(group_by(data2, Color1, Color2), n=n())
g2a$raw.weight <- temp$n[match(paste(g2a$Color1, g2a$Color2),  paste(temp$Color1, temp$Color2))]
g2w <- graph.data.frame(data.frame(Color1=g2a$Color1, Color2=g2a$Color2, weight=g2a$raw.weight), directed=F) # Network
head(data2);dim(data2)
(y2mod <- cluster_walktrap(g2w, steps=10, weights=E(g2w)$weight))
modularity(y2mod) # 0.73

yr <- '17-18'
data3 <- subset(ally, studyyear==yr)[,c('Color1','Color2','sitefix','date','Lek')]; dim(data3); length(table(data3$Lek))
g3a <- unique(data3[,c('Color1','Color2')])
g3a <- subset(g3a, Color1<Color2)
temp <- summarize(group_by(data3, Color1, Color2), n=n())
g3a$raw.weight <- temp$n[match(paste(g3a$Color1, g3a$Color2),  paste(temp$Color1, temp$Color2))]
g3w <- graph.data.frame(data.frame(Color1=g3a$Color1, Color2=g3a$Color2, weight=g3a$raw.weight), directed=F) # Network
head(data3);dim(data3)
(y3mod <- cluster_walktrap(g3w, steps=10, weights=E(g3w)$weight))
modularity(y3mod) # 0.83

# Compile membership data for each bird year, to compare lek membershp with cluster membership:
cdata <- data.frame(ID=c(names(V(g1w)), names(V(g2w)), names(V(g3w))), studyyear=c(rep('15-16',length(V(g1w))), rep('16-17',length(V(g2w))), rep('17-18',length(V(g3w)))), membership=c(y1mod$membership, y2mod$membership, y3mod$membership))
cdata$membership <- paste(cdata$studyyear, cdata$membership, sep='_') # Cluster membership
cdata$prim.Lek <-NA; cdata$prop <- NA # Determine primary lek:
for(i in 1:nrow(cdata)){
	yr <- cdata$studyyear[i]
	b1 <- cdata$ID[i]
	tab <- sort(table(subset(ally, Color1==b1 & studyyear==yr)$Lek), decreasing=T)
	cdata$prim.Lek[i] <- names(tab)[1]
	cdata$prop[i] <- tab[1]/sum(tab)
}
summary(cdata$prop) # Proportion of social interactions on a male's primary lek

# Histogram of primary lek usage:
dev.new(width=3, height=2) 
par(mar=c(3,3,0.1,0.1), las=1, bty='l', tck=-0.03, mgp=c(2,0.25,0))
hist(cdata$prop, breaks=25, xlab='Proportion', las=1, main='', xlim=c(0,1), col=rgb(0,0,1,0.5), border=NA, xaxt='n') # Cross-lek activity is rare
axis(1, at=c(0,0.5,1)) # Proportion of interactions on primary lek

# Set up colors for plotting:
gcols <- c('#FF7043', '#F06292', '#DB4437', '#B762C6', '#7481C9', '#4285F4', '#00ACC1', '#33AB71', '#F5BF26', '#ACAB44', '#9D794C')
tcols <- t(col2rgb(gcols, alpha=T))/255; tcols[,4] <- 0.5; 
ccols <- vector("list", length=11); ccolsv <- rep(NA, 11)
for(i in 1:11){
	ccols[[i]] <- tcols[i,] 
	ccolsv[i] <- rgb(ccols[[i]][1],ccols[[i]][2],ccols[[i]][3],ccols[[i]][4])
}
cdata$Lek.col <- gcols[as.numeric(factor(cdata$prim.Lek))] # [c(1,7,2,8,3,9,4,10,5,11,6)]

cdata$mnum <- as.numeric(stringr::str_split(cdata$membership, '_', simplify=T)[,2])
cdataO <- cdata[with(cdata,order(studyyear, mnum, prim.Lek)),] # ordered dataset
cdataO$clus.col <- cdataO$mnum
cdataO$clus.col <- ccolsv[as.numeric(cdataO$clus.col)] # [c(1,7,2,8,3,9,4,10,5,11,6)]
cdata$clus.col <- ccolsv[as.numeric(cdata$mnum)] # [c(1,7,2,8,3,9,4,10,5,11,6)]

dev.new(width=4, height=6)
par(mar=c(3,3,3,3));plot(NULL, bty='n', xaxt='n', yaxt='n', xlim=c(0,30), ylim=c(120,0)) # Plot correspondence clusters and leks
text(x=1, y=1:100, cdataO$ID[1:100], cex=0.35, col=cdataO$clus.col[1:100], pos=4)
points(c(1:100) ~ rep(5,100), pch=16, cex=0.5, col=cdataO$Lek.col[1:100])
segments(x0=1.75, x1=1.75, y0=cumsum(table(cdataO$mnum[1:100]))-table(cdataO$mnum[1:100])+1, y1=cumsum(table(cdataO$mnum[1:100])))

text(x=11, y=1:114, cdataO$ID[101:214], cex=0.35, col=cdataO$clus.col[101:214], pos=4)
points(c(1:114) ~ rep(15,114), pch=16, cex=0.5, col=cdataO$Lek.col[101:214])
segments(x0=11.75, x1=11.75, y0=cumsum(table(cdataO$mnum[101:214]))-table(cdataO$mnum[101:214])+1, y1=cumsum(table(cdataO$mnum[101:214])))

text(x=21, y=1:82, cdataO$ID[215:296], cex=0.35, col=cdataO$clus.col[215:296], pos=4)
points(c(1:82) ~ rep(25,82), pch=16, cex=0.5, col=cdataO$Lek.col[215:2960])
segments(x0=21.75, x1=21.75, y0=cumsum(table(cdataO$mnum[215:296]))-table(cdataO$mnum[215:296])+1, y1=cumsum(table(cdataO$mnum[215:296])))

leg.info <- unique(cdata[,c('prim.Lek','Lek.col')]); leg.info <- leg.info[with(leg.info, order(prim.Lek)),]
legend('bottomright', bty='n', col=leg.info$Lek.col, lwd=10, legend=leg.info$prim.Lek, cex=0.5)

dev.new(width=4, height=4)
par(mar=c(0,0,0,0)); plot(g3w, vertex.size=3, vertex.label=NA, vertex.color=cdataO$Lek.col[215:296][order(as.numeric(rownames(cdataO[215:296,])))], vertex.frame.color=rgb(0,0,0,0.3), edge.color=rgb(0,0,0,0.3), edge.width=log(edge_attr(g3w)$weight+.1, base=10)/2, layout=layout_with_lgl, mark.groups=y3mod, mark.expand=3, mark.col= ccolsv, mark.border=ccolsv)


# Sankey diagram:
(y3sankey <- summarize(group_by(subset(cdata, studyyear=='17-18'), membership, mnum, clus.col, prim.Lek, Lek.col), weight=n()))
y3sankey$mnum <- factor(y3sankey$mnum)

sk3 <- gvisSankey(data=y3sankey[,c('mnum','prim.Lek','weight')], from='mnum', to='prim.Lek', weight='weight',
						options=list(width=200))
plot(sk3) # Note the colors have to be edited manually to match the other key


# Descriptive statistics for the correspondence:
cdata$match <- NA; cdata$total <- NA; cdata$count <- NA
for(i in 1:nrow(cdata)){
	(temp <- subset(cdata, studyyear==cdata$studyyear[i] & membership==cdata$membership[i]))
	cdata$match[i] <- cdata$prim.Lek[i] == names(sort(table(temp$prim.Lek), decreasing=T)[1])	
	temp <- table(subset(ally, Color1==cdata$ID[i] & studyyear==cdata$studyyear[i])$Lek)
	cdata$total[i] <- sum(temp)
	cdata$count[i] <- temp[names(temp)==cdata$prim.Lek[i]]
}
summary(cdata$match); 266/296 # 266/296 = 90% match

# Binomial regression:
summary(mod <- glm(cbind(count, total-count) ~ factor(match), data=cdata, family='binomial')) # p < 0.0001
lsmeans::lsmeans(mod, 'match', transform='response') # 78% [76-80] vs. 98% [98-99] This analysis indicates that even the males with a lek-cluster mismatch interacted with 78% probability on their primary lek (least-squares mean). i.e., they did interact across lek boundaries but still relatively rarely, on average. For those with a match, the probability is 98%.

assortativity_nominal(g1w, types=factor(subset(cdata, studyyear=='15-16')$membership), directed=F) # 0.47
assortativity_nominal(g1w, types=factor(subset(cdata, studyyear=='15-16')$prim.Lek), directed=F) # 0.51

assortativity_nominal(g2w, types=factor(subset(cdata, studyyear=='16-17')$membership), directed=F) # 0.52
assortativity_nominal(g2w, types=factor(subset(cdata, studyyear=='16-17')$prim.Lek), directed=F) # 0.51

assortativity_nominal(g3w, types=factor(subset(cdata, studyyear=='17-18')$membership), directed=F) # 0.69
assortativity_nominal(g3w, types=factor(subset(cdata, studyyear=='17-18')$prim.Lek), directed=F) # 0.69


g1mat <- as_adj(g1w, sparse=F, attr='weight') # Weighted assortment is even higher:
(g1_m_assort <- assortnet::assortment.discrete(g1mat, types=factor(subset(cdata, studyyear=='15-16')$membership), weighted=TRUE)$r)
(g1_lk_assort <- assortnet::assortment.discrete(g1mat, types=factor(subset(cdata, studyyear=='15-16')$prim.Lek), weighted=TRUE)$r)

g2mat <- as_adj(g2w, sparse=F, attr='weight') 
(g2_m_assort <- assortnet::assortment.discrete(g2mat, types=factor(subset(cdata, studyyear=='16-17')$membership), weighted=TRUE)$r)
(g2_lk_assort <- assortnet::assortment.discrete(g1mat, types=factor(subset(cdata, studyyear=='15-16')$prim.Lek), weighted=TRUE)$r)

g3mat <- as_adj(g3w, sparse=F, attr='weight') 
(g3_m_assort <- assortnet::assortment.discrete(g3mat, types=factor(subset(cdata, studyyear=='17-18')$membership), weighted=TRUE)$r)
(g3_lk_assort <- assortnet::assortment.discrete(g1mat, types=factor(subset(cdata, studyyear=='15-16')$prim.Lek), weighted=TRUE)$r)










