
# Reciprocity and behavioral heterogeneity govern the stability of social networks 
# Roslyn Dakin and T. Brandt Ryder

# This script plots simple, visualized examples of the simulation for the supplement figure
# Extended Data Figure 2

library(igraph)
library(dplyr)

new.dat <- data.frame(nbird=rep(11:20, each=15*20), target.density=rep(seq(0.2,0.8, length.out=20), each=15), weight.factor=seq(0.2,2,length.out=15), network.weight=NA, density=NA, cv_deg=NA, stability.edge.model=NA, stability.individual.model=NA); head(new.dat); dim(new.dat)
(plot.examples <- subset(new.dat, nbird==11 & round(target.density,2) %in% c(0.36, 0.67) & round(weight.factor,2) %in% c(1.49, 2.0)))
plot.examples$nbird <- 8 # Although the smallest simulaton had a network size of 11, here we use 8 for ease of visually interpretation

store.plots <- vector('list', length=nrow(plot.examples)) # Set up the list to store the simulated networks
for(i in 1:length(store.plots)){
	store.plots[[i]] <- vector('list', length=7) # 4 examples, each of which will have 7 networks to illustrate
}

# Function to create a network at a given size and target density:
net_create <- function(gsize=20, target.density=0.5){ 
	g <- sample_smallworld(dim=1, size=gsize, nei=gsize-1, p=0, loops=F, multiple=F) # Start with a completely connected graph
	dcorrect <- edge_density(g) - target.density # Prune to target density:
	if(dcorrect>0 & round(abs(dcorrect)*choose(length(V(g)), 2)) >0){
		g <- delete_edges(g, edges=sample(E(g), size=round(abs(dcorrect)*choose(length(V(g)), 2)), replace=F))
	}
	return(g)	
}

load('edge_level_predictions.RData'); ls() # Load this to get the observed edge weights

for(i in 1:nrow(plot.examples)){
	
	set.seed(i+1)
	
	net <- net_create(gsize=plot.examples$nbird[i], target.density=plot.examples$target.density[i]) # Starting network
	
	# Apply edge weights sampling from manakin data:
	g1b <- igraph::as_data_frame(net)
	g1b$raw.weight <- sample(allweights[allweights>0 & allweights<501], nrow(g1b))
	g1b$raw.weight <- ceiling(plot.examples$weight.factor[i]*(g1b$raw.weight)) # Apply the weight factor
	g1b$raw.weight[g1b$raw.weight>500] <- 500 # Truncate at 500
	
	# Get network properties:
	plot.examples$network.weight[i] <- mean(log(g1b$raw.weight))
	plot.examples$density[i] <- edge_density(net)
	degs <- log(igraph::degree(net))
	plot.examples$cv_deg[i] <- sd(degs[degs!=-Inf]) / mean(degs[degs!=-Inf])

	# Now, run the individual-based simulation:

	net.mat <- as_adj(net, sparse=F)
	for(j in 1:nrow(g1b)){ # Add edge weights
		net.mat[g1b$from[j], g1b$to[j]] <- g1b$raw.weight[j]
		net.mat[g1b$to[j], g1b$from[j]] <- g1b$raw.weight[j]
	}
	new.mat <- net.mat # net.mat is the starting network
	new.mat[] <- 0 # new.mat will store each time step
	cum.mat <- new.mat # cum.mat will sum the interactions (network) resulting from simulated behavior

	nattempts <- ceiling(log(rowSums(net.mat)+1, base=10) + 0.1) # Heterogeneity assumption
	# Each individual's no. of interaction attempts per time step is scaled to his activity in the starting network.

	nit <- 5 # time steps

	store.plots[[i]][[1]] <- graph_from_adjacency_matrix(net.mat, mode='undirected', weighted=T) # Starting network
	
	for(k in 1:nit){
		for(j in 1:nrow(new.mat)){
			pot.ps <- which(net.mat[j,]>0)
			if(length(pot.ps)<1){
				pot.ps <- sample((1:ncol(new.mat))[-j], 1)
			}
			if(length(pot.ps)>1){
				pot.ps <- pot.ps[order(net.mat[j,][which(net.mat[j,]>0)], decreasing=T)]
				if(length(pot.ps) < nrow(net.mat)-1){
					pot.ps <- c(pot.ps, sample((1:ncol(new.mat))[!(1:ncol(new.mat)) %in% c(j, pot.ps)], 1))
				}
				new.mat[j, sample(pot.ps, nattempts[j], replace=T, prob=length(pot.ps):1)] <- 0.5
			} else {
				new.mat[j, pot.ps] <- 0.5
			}
		}
		new.mat[upper.tri(new.mat)] <- new.mat[upper.tri(new.mat)] + t(new.mat)[upper.tri(t(new.mat))]
		new.mat[] <- floor(new.mat[])
		new.mat[lower.tri(new.mat)] <- t(new.mat)[lower.tri(t(new.mat))]
		cum.mat <- new.mat + cum.mat
		store.plots[[i]][[k+1]] <- graph_from_adjacency_matrix(new.mat, mode='undirected', weighted=T) # store each time step
		new.mat[] <- 0
	}

	n.dat <- igraph::as_data_frame(graph.adjacency(cum.mat>0, mode='undirected'))
	
	store.plots[[i]][[7]] <- graph_from_adjacency_matrix(cum.mat, mode='undirected', weighted=T) # final network
	
	# Calculate stability:
	plot.examples$stability.individual.model[i] <- length(intersect(unique(c(paste(n.dat$from,n.dat$to), paste(n.dat$to, n.dat$from))), unique(c(paste(g1b$from,g1b$to), paste(g1b$to, g1b$from))))) / length(union(unique(c(paste(n.dat$from,n.dat$to), paste(n.dat$to, n.dat$from))), unique(c(paste(g1b$from,g1b$to), paste(g1b$to, g1b$from)))))
		
	print(i)
}
plot.examples

# Layout for plotting:
(mymat <- matrix(data=c(rep(c(0,0,0,0,0,2,3,4,5,6,0,0,0,0,0), each=2), rep(c(1,7), each=15), rep(c(1,7), each=15), rep(c(1,7), each=15)), nrow=4, ncol=30, byrow=T))

# Plot the 4 examples showing initial network, time steps, and final network (sum of the time steps)
for(j in 1:4){
	dev.new(width=4, height=2)
	par(mar=c(0.2,0.2,0.2,0.2))
	layout(mat=mymat)
	for(i in 1:7){
		plot(store.plots[[j]][[i]], layout=layout_in_circle, edge.width=log(edge_attr(store.plots[[j]][[i]])$weight+1), vertex.label=NA, vertex.color='light blue', vertex.frame.color='light blue')
		
	}
	text(0,0,round(plot.examples$stability.individual.model[j],2))
}








