
# Reciprocity and behavioral heterogeneity govern the stability of social networks 
# Roslyn Dakin and T. Brandt Ryder

# This script generates the individual-based simulation models
# Figures 2, 3, 4

library(igraph)
library(dplyr)
library(lme4)
library(lmerTest)
library(visreg)

# Get social interaction data:
year3 <- read.csv('filtered_interactions_nodups_times_durations_052418.csv', stringsAsFactors=F)[,c('date','nTime','Color1','Color2','Lek','studyyear','DL','sitefix')]; dim(year3)
year12 <- read.csv('filtered_interactions_nodups_times_durations_011018.csv', stringsAsFactors=F)[,c('date','nTime','Color1','Color2','Lek','studyyear','DL','sitefix')]; dim(year12)
ally <- rbind(year12, year3); head(ally); dim(ally) # Duplicate and reverse individual 1 & individual 2 to account for all interactions:
allyX <- ally
allyX$Color1 <- ally$Color2; allyX$Color2 <- ally$Color1
dim(ally); dim(allyX)
ally <- rbind(ally, allyX)
ally$Color1[ally$Color1=='OX-'] <- 'OX' # Correct ID error
ally$Color2[ally$Color2=='OX-'] <- 'OX'
ally$sitefix[ally$sitefix=='OX-'] <- 'OX'
all(levels(factor(ally$Color1))==levels(factor(ally$Color2))); dim(ally) # 73,770

# Dataframe to store outcomes of the simulation:
new.dat <- data.frame(nbird=rep(11:20, each=15*20), target.density=rep(seq(0.2,0.8, length.out=20), each=15), weight.factor=seq(0.2,2,length.out=15), network.weight=NA, density=NA, cv_deg=NA, stability.edge.model=NA, stability.individual.model=NA); head(new.dat); dim(new.dat)

# We will initiate 3,000 starting networks that display a uniform distribution of the three network properties of interest: size, weight, and density.
table(new.dat$nbird, round(new.dat$target.density,2))
table(new.dat$nbird, round(new.dat$weight.factor,2))
table(round(new.dat$weight.factor,2), round(new.dat$target.density,2))

# Here, we will store the edge-level results of the individual-based behavioral simulation. Data will grow with the simulation because the exact number of edges is not pre-determined:
edge.dat <- data.frame(raw.weight=NA, edge.connectivity=NA, persistence.individual.model=NA, index=NA)

load('edge_level_predictions.RData'); ls() # Load predictions from the MMM model at the edge-level. These will be used for a concurrent edge-based simulaton.
head(pred.data) # "pred" is the predicted posterior probabilities that a partnership will persist, given a range of parametres, from the fitted MMM model at the edge-level
head(allweights) # Vector of observed annual interaction frequencies (edge weights) to parameterize edge weights of the simulated networks.

# Function to create a network at a given size and target density:
net_create <- function(gsize=20, target.density=0.5){ 
	g <- sample_smallworld(dim=1, size=gsize, nei=gsize-1, p=0, loops=F, multiple=F) # Start with a completely connected graph
	dcorrect <- edge_density(g) - target.density # Prune to target density:
	if(dcorrect>0 & round(abs(dcorrect)*choose(length(V(g)), 2)) >0){
		g <- delete_edges(g, edges=sample(E(g), size=round(abs(dcorrect)*choose(length(V(g)), 2)), replace=F))
	}
	return(g)	
}

# Run the simulation:
for(i in 1:nrow(new.dat)){
	
	set.seed(i)
	
	# Create the starting network:
	net <- net_create(gsize=new.dat$nbird[i], target.density=new.dat$target.density[i])
	
	# Apply edge weights sampling from manakin data:
	g1b <- igraph::as_data_frame(net)
	g1b$raw.weight <- sample(allweights[allweights>0 & allweights<501], nrow(g1b))
	g1b$raw.weight <- ceiling(new.dat$weight.factor[i]*(g1b$raw.weight)) # Apply the weight factor to generate additional variance in network weight, independent of density:
	g1b$raw.weight[g1b$raw.weight>500] <- 500 # Truncate at 500
	
	# Get network properties:
	new.dat$network.weight[i] <- mean(log(g1b$raw.weight))
	new.dat$density[i] <- edge_density(net)
	degs <- log(igraph::degree(net))
	new.dat$cv_deg[i] <- sd(degs[degs!=-Inf]) / mean(degs[degs!=-Inf])
	
	g1b$edge.connectivity <- NA; g1b$probability <- NA; g1b$persists <- NA # Here, we use the empirical results to concurrently run an edge-based simulation. This is not currently presented in the manuscript, but has network-level results consistent with the observed data.
	for(j in 1:nrow(g1b)){
		g1b$edge.connectivity[j] <- edge_connectivity(net, source=g1b$from[j], target=g1b$to[j])
		g1b$probability[j] <- subset(pred.data, edge.weight==g1b$raw.weight[j] & connect==g1b$edge.connectivity[j])$pred # Predicted probability of edge persistence based on manakin edge-level data.
		g1b$persists[j] <- sample(c(0,1), size=1, prob=c(1-g1b$probability[j], g1b$probability[j])) # Outcome determined from the predicted probability
	}
	new.dat$stability.edge.model[i] <- sum(g1b$persists)/nrow(g1b) # Corresponds to stability in the absence of new edge creation, in this edge-based simulation
	
	# Now, run the individual-based simulation:

	net.mat <- as_adj(net, sparse=F)
	for(j in 1:nrow(g1b)){ # Add edge weights
		net.mat[g1b$from[j], g1b$to[j]] <- g1b$raw.weight[j]
		net.mat[g1b$to[j], g1b$from[j]] <- g1b$raw.weight[j]
	}
	new.mat <- net.mat # net.mat is the starting network
	new.mat[] <- 0 # new.mat will store each time step
	cum.mat <- new.mat # cum.mat will sum the interactions (network) resulting from simulated behavior

	nattempts <- ceiling(log(rowSums(net.mat)+1, base=10) + 0.1) # Among-individual heterogeneity assumption
	# Each individual's no. of interaction attempts per time step is scaled to his activity in the starting network.
	
	nit <- 5 # time steps

	for(k in 1:nit){
		for(j in 1:nrow(new.mat)){
			pot.ps <- which(net.mat[j,]>0) # ID of potential partners, based on assumed preference for previous partners (part of reciprocity assumption)
			if(length(pot.ps)<1){
				pot.ps <- sample((1:ncol(new.mat))[-j], 1) # If there are any unconnected individuals, they choose randomly
			}
			if(length(pot.ps)>1){
				pot.ps <- pot.ps[order(net.mat[j,][which(net.mat[j,]>0)], decreasing=T)]
				if(length(pot.ps) < nrow(net.mat)-1){
					pot.ps <- c(pot.ps, sample((1:ncol(new.mat))[!(1:ncol(new.mat)) %in% c(j, pot.ps)], 1))
				}
				new.mat[j, sample(pot.ps, nattempts[j], replace=T, prob=length(pot.ps):1)] <- 0.5 # Each attempt gets 0.5, stored in new.mat. Both partners must attempt (0.5*2 = 1) to be sufficident for a successful, reciprocated interaction.
			} else {
				new.mat[j, pot.ps] <- 0.5
			}
		}
		new.mat[upper.tri(new.mat)] <- new.mat[upper.tri(new.mat)] + t(new.mat)[upper.tri(t(new.mat))] # Sum to get outcome
		new.mat[] <- floor(new.mat[]) # Eliminate unsuccessful attempts (not reciprocated)
		new.mat[lower.tri(new.mat)] <- t(new.mat)[lower.tri(t(new.mat))]
		cum.mat <- new.mat + cum.mat # Add to cum.mat
		new.mat[] <- 0 # Clear for next time step
	}

	n.dat <- igraph::as_data_frame(graph.adjacency(cum.mat>0, mode='undirected'))
	
	# Calculate stability:
	new.dat$stability.individual.model[i] <- length(intersect(unique(c(paste(n.dat$from,n.dat$to), paste(n.dat$to, n.dat$from))), unique(c(paste(g1b$from,g1b$to), paste(g1b$to, g1b$from))))) / length(union(unique(c(paste(n.dat$from,n.dat$to), paste(n.dat$to, n.dat$from))), unique(c(paste(g1b$from,g1b$to), paste(g1b$to, g1b$from)))))
	
	# Edge-level outcome of the individual-based simulation:
	g1b$persistence.individual.model <- paste(g1b$from, g1b$to) %in% paste(n.dat$from, n.dat$to)
	g1b$index <- i
	edge.dat <- rbind(edge.dat, g1b[,c('raw.weight', 'edge.connectivity', 'persistence.individual.model', 'index')])
	
	print(i)
}
summary(new.dat)

dev.new(width=3.5, height=3.5)
par(mfrow=c(2,2), las=1, bty='l', mar=c(4,4,0.5,0.5), cex.axis=0.75, cex.lab=0.75, mgp=c(2,0.5,0))
plot(density  ~ target.density, new.dat, pch=16, cex=0.5, col=rgb(0,0,0,0.1))
plot(network.weight  ~ weight.factor, new.dat, pch=16, cex=0.5, col=rgb(0,0,0,0.1))
plot(stability.individual.model  ~ stability.edge.model, new.dat, pch=16, cex=0.5, col=rgb(0,0,0,0.1)); abline(a=0,b=1) # Strong correspondence between outcome of individual-based and edge-based models

# Function to plot 95% CIS of the partial residuals binned along the x-axis, for analysis scatterplots:
mypolyfun <- function(object, variable, p.col=rgb(0,0,0,0.1)){ 
	# object<- b; variable<- 'mean.ew'; p.col <- rgb(0,0,0,0.1)
	temp <- object$res
	temp$cat <- ggplot2::cut_number(temp[,variable], n=100)
	poly <- data.frame(x=sort(unique(temp$cat)), lower=NA, upper=NA, x2=NA)
	for(i in 1:nrow(poly)){
		poly$lower[i] <- quantile(subset(temp, cat == poly$x[i])$visregRes, 0.025)
		poly$upper[i] <- quantile(subset(temp, cat == poly$x[i])$visregRes, 0.975)
		poly$x2[i] <- mean(subset(temp, cat == poly$x[i])[,variable])
	}
	polygon(x=c(poly$x2, rev(poly$x2)), y=c(poly$lower, rev(poly$upper)), border=NA, col=p.col)
}

# Statistical analysis of the data from the main, individual-based model:
summary(mod <- lm(scale(stability.individual.model)~ scale(log(nbird)) + scale(network.weight) + scale(density), data=new.dat)) # Standardized for summary
mod <- lm(stability.individual.model ~ log(nbird) + network.weight + density, data=new.dat) # Unstandardized for plotting

b <- visreg(mod, 'network.weight', plot=F)
c <- visreg(mod, 'density', plot=F)

dev.new(width=4.5, height=2) # Here, we plot the results of the individual-based model of reciprocity and heterogeneity:
par(mfrow=c(1,2), las=1, bty='l', mar=c(3,3,0.2,0.2), mgp=c(1.5,0.25,0), tck=-0.03)
plot(visregRes ~ network.weight, data=b$res, pch=16, col=rgb(0,0,0,0.05), cex=0.5, ylim=c(0,1), ylab='Network stability', xlab='Network weight', xlim=c(0.4,2.65), type='n')
points(visregFit ~ network.weight, data=b$fit, type='l', lwd=2.5)
mypolyfun(object=b, variable='network.weight')
plot(visregRes ~ density, data=c$res, pch=16, col=rgb(0,0,0,0.05), cex=0.5, ylim=c(0,1), ylab='', xlab='Network density', xlim=c(0.13,0.82), type='n')
points(visregFit ~ density, data=c$fit, type='l', lwd=2.5)
mypolyfun(object=c, variable='density')

# Here, we alxo consider the effect of initial variation in CVdegree (degree heterogeneity among individuals): 
summary(mod <- lm(stability.individual.model ~ log(nbird) + network.weight + density + cv_deg, data=new.dat))
# Note that size, weight, and density alone explain over 80% of the total variance, because the starting networks were designed to vary in these parameters; CVdegree has less variation independent of these other factors, but still enough for an exploratory analysis here:

d <- visreg(mod, 'cv_deg', plot=F)

dev.new(width=4.5, height=2) 
par(mfrow=c(1,2), las=1, bty='l', mar=c(3,3,0.2,0.2), mgp=c(1.5,0.25,0), tck=-0.03)
plot(visregRes ~ cv_deg, data=d$res, pch=16, col=rgb(0,0,0,0.05), cex=0.5, ylim=c(0.225,0.7), ylab='Network stability', xlab='CVdegree', type='n', xlim=c(0,0.8))
points(visregFit ~ cv_deg, data=d$fit, type='l', lwd=2.5)
mypolyfun(object=d, variable='cv_deg')
plot(visregRes ~ cv_deg, data=d$res, pch=16, col=rgb(0,0,0,0.05), cex=0.5, ylim=c(0.225,0.7), ylab='Network stability', xlab='CVdegree', type='p', xlim=c(0,0.8))
points(visregFit ~ cv_deg, data=d$fit, type='l', lwd=2.5)
mypolyfun(object=d, variable='cv_deg')


# Here, we look at the network-level outcome of a simulation parameterized using our edge-level empirical analysis: 
summary(mod <- lm(scale(stability.edge.model) ~ scale(log(nbird)) + scale(network.weight) + scale(density), data=new.dat)) 
(mod <- lm((stability.edge.model)~ scale(log(nbird)) + scale(network.weight) + scale(density), data=new.dat)) 
# save(mod, file='individual_based_simulation_fitted_regression_model.RData')

b <- visreg(mod, 'network.weight', plot=F)
c <- visreg(mod, 'density', plot=F)

dev.new(width=4.5, height=2) # Plotted, it is highly consistent with the individual-based simulation (reciproxity + heterogeneity)
par(mfrow=c(1,2), las=1, bty='l', mar=c(3,3,0.2,0.2), mgp=c(1.5,0.25,0), tck=-0.03)
plot(visregRes ~ network.weight, data=b$res, pch=16, col=rgb(0,0,0,0.05), cex=0.5, ylim=c(0,1), ylab='Network stability', xlab='Network weight', xlim=c(0.4,2.65), type='n')
points(visregFit ~ network.weight, data=b$fit, type='l', lwd=2.5)
mypolyfun(object=b, variable='network.weight')
plot(visregRes ~ density, data=c$res, pch=16, col=rgb(0,0,0,0.05), cex=0.5, ylim=c(0,1), ylab='', xlab='Network density', xlim=c(0.13,0.82), type='n')
points(visregFit ~ density, data=c$fit, type='l', lwd=2.5)
mypolyfun(object=c, variable='density')



# Here, we examine results at the edge level based on the individual-based behavioral simulation:

set.seed(101); index.sample <- sample(1:3000, 86) # Because the entire data set is too large to compute the model fit, we examine a random subsetL
summary(mod <- glmer(persistence.individual.model ~ scale(log(raw.weight)) + scale(log(edge.connectivity)) + (1|index), data=subset(edge.dat, !is.na(index) & index %in% index.sample), family='binomial') )

dev.new(width=4.5, height=2) # Plot the predictions of the analysis, for comparison with empirial edge-level results:
par(mfrow=c(1,2), las=1, bty='l', mar=c(3,3,0.2,0.2), mgp=c(1.5,0.25,0), tck=-0.03)
plot(predict(mod, type='response') ~ log(subset(edge.dat, !is.na(index) & index %in% index.sample)$raw.weight), pch=16, col=rgb(0,0,0,0.05), cex=0.5, ylim=c(0,1), xlim=c(0,6.22), ylab='Partnership persistence', xlab='Edge weight (log)', type='n')
points(predict(mod, type='response', newdata=data.frame(raw.weight=seq(1,470,by=0.1), edge.connectivity=8, index=index.sample[1])) ~ log(seq(1,470,by=0.1)), type='l', lwd=2.5)
polygon(x=c(log(seq(1,470,by=0.1)), rev(log(seq(1,470,by=0.1)))), y=c(predict(mod, type='response', newdata=data.frame(raw.weight=seq(1,470,by=0.1), edge.connectivity=3, index=index.sample[1])), rev(predict(mod, type='response', newdata=data.frame(raw.weight=seq(1,470,by=0.1), edge.connectivity=14, index=index.sample[1])))), border=NA, col=rgb(0,0,1,0.1)) # using mean and 5th, 95th percentiles from simulation data
plot(predict(mod, type='response') ~ log(subset(edge.dat, !is.na(index) & index %in% index.sample)$edge.connectivity), pch=16, col=rgb(0,0,0,0.05), cex=0.5, ylim=c(0,1), xlim=c(1.10, 3.64), ylab='Partnership persistence', xlab='Edge connectivity (log)', type='n')
points(predict(mod, type='response', newdata=data.frame(raw.weight=15, edge.connectivity=seq(3,38,by=0.01), index=index.sample[1])) ~ log(seq(3,38,by=0.01)), type='l', lwd=2.5)
polygon(x=c(log(seq(3,38,by=0.01)), rev(log(seq(3,38,by=0.01)))), y=c(predict(mod, type='response', newdata=data.frame(raw.weight=1, edge.connectivity=seq(3,38,by=0.01), index=index.sample[1])), rev(predict(mod, type='response', newdata=data.frame(raw.weight=109, edge.connectivity=seq(3,38,by=0.01), index=index.sample[1])))), border=NA, col=rgb(0,0,1,0.1)) # using mean and 5th, 95th percentiles from simulation data







