
# Reciprocity and behavioral heterogeneity govern the stability of social networks 
# Roslyn Dakin and T. Brandt Ryder

# This script generates various null simulation models for comparison with the individual-based simulation model
# Figure 2, Extended Data Tables

library(igraph)
library(dplyr)
library(lme4)
library(lmerTest)
library(visreg)

load('edge_level_predictions.RData'); head(allweights) # Load observed edge weights

# Function to create a network at a given size and target density:
net_create <- function(gsize=20, target.density=0.5){ 
	g <- sample_smallworld(dim=1, size=gsize, nei=gsize-1, p=0, loops=F, multiple=F) # Start with a completely connected graph
	dcorrect <- edge_density(g) - target.density # Prune to target density:
	if(dcorrect>0 & round(abs(dcorrect)*choose(length(V(g)), 2)) >0){
		g <- delete_edges(g, edges=sample(E(g), size=round(abs(dcorrect)*choose(length(V(g)), 2)), replace=F))
	}
	return(g)	
}

# Function to plot 95% CIS of the partial residuals binned along the x-axis, for analysis scatterplots:
mypolyfun <- function(object, variable, p.col=rgb(0,0,0,0.1)){ 
	# object<- b; variable<- 'mean.ew'; p.col <- rgb(0,0,0,0.1)
	temp <- object$res
	temp$cat <- ggplot2::cut_number(temp[,variable], n=100)
	poly <- data.frame(x=sort(unique(temp$cat)), lower=NA, upper=NA, x2=NA)
	for(i in 1:nrow(poly)){
		poly$lower[i] <- quantile(subset(temp, cat == poly$x[i])$visregRes, 0.025)
		poly$upper[i] <- quantile(subset(temp, cat == poly$x[i])$visregRes, 0.975)
		poly$x2[i] <- mean(subset(temp, cat == poly$x[i])[,variable])
	}
	polygon(x=c(poly$x2, rev(poly$x2)), y=c(poly$lower, rev(poly$upper)), border=NA, col=p.col)
}




# NULL 1 randomly rewire the network (preserving density); i.e., simplest null, and not an individual-based model:
#####################################################################################################################

new.dat_NULL1_rewire <- data.frame(nbird=rep(11:20, each=15*20), target.density=rep(seq(0.2,0.8, length.out=20), each=15), weight.factor=seq(0.2,2,length.out=15), network.weight=NA, density=NA, stability=NA); dim(new.dat_NULL1_rewire); head(new.dat_NULL1_rewire)

for(i in 1:nrow(new.dat_NULL1_rewire)){
	
	set.seed(i)
	
	# Create the starting network:
	net <- net_create(gsize=new.dat_NULL1_rewire$nbird[i], target.density=new.dat_NULL1_rewire$target.density[i])
		
	# Apply edge weights sampling from manakin data:
	g1b <- igraph::as_data_frame(net)
	g1b$raw.weight <- sample(allweights[allweights>0 & allweights<501], nrow(g1b))
	g1b$raw.weight <- ceiling(new.dat_NULL1_rewire$weight.factor[i]*(g1b$raw.weight)) # Apply the weight factor:
	g1b$raw.weight[g1b$raw.weight>500] <- 500 # Truncate at 500
	
	# Get network properties:
	new.dat_NULL1_rewire$network.weight[i] <- mean(log(g1b$raw.weight))
	new.dat_NULL1_rewire$density[i] <- edge_density(net)
	
	# Run the simulation:
	net.mat <- as_adj(net, sparse=F)
	for(j in 1:nrow(g1b)){
		net.mat[g1b$from[j], g1b$to[j]] <- g1b$raw.weight[j]
		net.mat[g1b$to[j], g1b$from[j]] <- g1b$raw.weight[j]
	}
	new.mat <- net.mat
	new.mat[] <- 0
	cum.mat <- new.mat
	
	new.mat[upper.tri(new.mat)] <- sample(net.mat[upper.tri(net.mat)]) # Random rewiring here, keeping density same, no need for iterations
	new.mat[lower.tri(new.mat)] <- t(new.mat)[lower.tri(t(new.mat))]
	cum.mat <- new.mat

	n.dat <- igraph::as_data_frame(graph.adjacency(cum.mat>0, mode='undirected'))

	# Calculate stability:
	new.dat_NULL1_rewire$stability[i] <- length(intersect(unique(c(paste(n.dat$from,n.dat$to), paste(n.dat$to, n.dat$from))), unique(c(paste(g1b$from,g1b$to), paste(g1b$to, g1b$from))))) / length(union(unique(c(paste(n.dat$from,n.dat$to), paste(n.dat$to, n.dat$from))), unique(c(paste(g1b$from,g1b$to), paste(g1b$to, g1b$from)))))
	
	print(i)
}
summary(new.dat_NULL1_rewire)

sumN <- summary(mod <- lm(scale(stability) ~ scale(log(nbird)) + scale(network.weight) + scale(density), data=new.dat_NULL1_rewire)) 
sumN <- as.data.frame(coef(sumN)); sumN
mod <- lm(stability ~ log(nbird) + network.weight + density, data=new.dat_NULL1_rewire) # Unstandardized for plotting

b <- visreg(mod, 'network.weight', plot=F) # Plot of this rewiring null not used:
c <- visreg(mod, 'density', plot=F)

dev.new(width=4.5, height=2) # 
par(mfrow=c(1,2), las=1, bty='l', mar=c(3,3,0.2,0.2), mgp=c(1.5,0.25,0), tck=-0.03)

plot(visregRes ~ network.weight, data=b$res, pch=16, col=rgb(0,0,0,0.025), cex=1, ylim=c(0,1), ylab='Network stability', xlab='Network weight', xlim=c(0.4,2.65), type='n')
points(visregFit ~ network.weight, data=b$fit, type='l', lwd=2.5)
mypolyfun(object=b, variable='network.weight')

plot(visregRes ~ density, data=c$res, pch=16, col=rgb(0,0,0,0.025), cex=1, ylim=c(0,1), ylab='', xlab='Network density', xlim=c(0.13,0.82), type='n')
points(visregFit ~ density, data=c$fit, type='l', lwd=2.5)
mypolyfun(object=c, variable='density')




# NULL 2 a null individual-based model, preserves the process by which network is rebuilt, but without reciprocity/heterogeneity:
######################################################################################################################################

new.dat_NULL2_ibm <- data.frame(nbird=rep(11:20, each=15*20), target.density=rep(seq(0.2,0.8, length.out=20), each=15), weight.factor=seq(0.2,2,length.out=15), network.weight=NA, density=NA, stability=NA); dim(new.dat_NULL2_ibm)

for(i in 1:nrow(new.dat_NULL2_ibm)){

	set.seed(i)
	
	# Create the starting network:
	net <- net_create(gsize=new.dat_NULL2_ibm$nbird[i], target.density=new.dat_NULL2_ibm$target.density[i])
	
	# Apply edge weights sampling from manakin data:
	g1b <- igraph::as_data_frame(net)
	g1b$raw.weight <- sample(allweights[allweights>0 & allweights<501], nrow(g1b))
	g1b$raw.weight <- ceiling(new.dat_NULL2_ibm$weight.factor[i]*(g1b$raw.weight)) # Apply the weight factor:
	g1b$raw.weight[g1b$raw.weight>500] <- 500 # Truncate at 500
	
	# Get network properties:
	new.dat_NULL2_ibm$network.weight[i] <- mean(log(g1b$raw.weight))
	new.dat_NULL2_ibm$density[i] <- edge_density(net)
	
	# Run the simulation:
	net.mat <- as_adj(net, sparse=F)
	for(j in 1:nrow(g1b)){
		net.mat[g1b$from[j], g1b$to[j]] <- g1b$raw.weight[j]
		net.mat[g1b$to[j], g1b$from[j]] <- g1b$raw.weight[j]
	}
	new.mat <- net.mat
	new.mat[] <- 0
	cum.mat <- new.mat

	nattempts <- rep(1, ncol(net.mat)) # Remove heterogeneity assumption, limit each individual to only one attempt

	nit <- 5 # time steps
	
	for(k in 1:nit){
		for(j in 1:nrow(new.mat)){
			pot.ps <- sample(1:ncol(net.mat)) # Removed assumption of stable preferences; all individuals have random preferences
			if(length(pot.ps)>1){
				new.mat[j, sample(pot.ps, nattempts[j], replace=T)] <- 1 # Removed requirement for reciprocation
			} else {
				new.mat[j, pot.ps] <- 1 # Removed requirement for reciprocation
			}
		}
		new.mat[upper.tri(new.mat)] <- new.mat[upper.tri(new.mat)] + t(new.mat)[upper.tri(t(new.mat))]
		new.mat[] <- floor(new.mat[])
		new.mat[lower.tri(new.mat)] <- t(new.mat)[lower.tri(t(new.mat))]
		cum.mat <- new.mat + cum.mat
		new.mat[] <- 0
	}

	n.dat <- igraph::as_data_frame(graph.adjacency(cum.mat>0, mode='undirected'))

	# Calculate stability:
	new.dat_NULL2_ibm$stability[i] <- length(intersect(unique(c(paste(n.dat$from,n.dat$to), paste(n.dat$to, n.dat$from))), unique(c(paste(g1b$from,g1b$to), paste(g1b$to, g1b$from))))) / length(union(unique(c(paste(n.dat$from,n.dat$to), paste(n.dat$to, n.dat$from))), unique(c(paste(g1b$from,g1b$to), paste(g1b$to, g1b$from)))))
	
	print(i)
}
summary(new.dat_NULL2_ibm)

sumN2 <- summary(mod <- lm(scale(stability)~ scale(log(nbird)) + scale(network.weight) + scale(density), data=new.dat_NULL2_ibm)) 
sumN2 <- as.data.frame(coef(sumN2)); sumN2
mod <- lm(stability ~ log(nbird) + network.weight + density, data=new.dat_NULL2_ibm) # Unstandardized for plotting

b <- visreg(mod, 'network.weight', plot=F)
c <- visreg(mod, 'density', plot=F)

dev.new(width=4.5, height=2) # 
par(mfrow=c(1,2), las=1, bty='l', mar=c(3,3,0.2,0.2), mgp=c(1.5,0.25,0), tck=-0.03)

plot(visregRes ~ network.weight, data=b$res, pch=16, col=rgb(0,0,0,0.025), cex=1, ylim=c(0,1), ylab='Network stability', xlab='Network weight', xlim=c(0.4,2.65), type='n')
points(visregFit ~ network.weight, data=b$fit, type='l', lwd=2.5)
mypolyfun(object=b, variable='network.weight')

plot(visregRes ~ density, data=c$res, pch=16, col=rgb(0,0,0,0.025), cex=1, ylim=c(0,1), ylab='', xlab='Network density', xlim=c(0.13,0.82), type='n')
points(visregFit ~ density, data=c$fit, type='l', lwd=2.5)
mypolyfun(object=c, variable='density')




# Null 2a this model assumes stable preferences ONLY, but removes heterogeneity and requirement for recprication to form a partnership:
######################################################################################################################################

new.dat_pref_only <- data.frame(nbird=rep(11:20, each=15*20), target.density=rep(seq(0.2,0.8, length.out=20), each=15), weight.factor=seq(0.2,2,length.out=15), network.weight=NA, density=NA, stability=NA); dim(new.dat_pref_only)

for(i in 1:nrow(new.dat_pref_only)){

	set.seed(i)

	# Create the starting network:
	net <- net_create(gsize=new.dat_pref_only$nbird[i], target.density=new.dat_pref_only$target.density[i])
	
	# Apply edge weights sampling from manakin data:
	g1b <- igraph::as_data_frame(net)
	g1b$raw.weight <- sample(allweights[allweights>0 & allweights<501], nrow(g1b))
	g1b$raw.weight <- ceiling(new.dat_pref_only$weight.factor[i]*(g1b$raw.weight)) # Apply the weight factor:
	g1b$raw.weight[g1b$raw.weight>500] <- 500 # Truncate at 500
	
	# Get network properties:
	new.dat_pref_only$network.weight[i] <- mean(log(g1b$raw.weight))
	new.dat_pref_only$density[i] <- edge_density(net)

	# Run the simulation:	
	net.mat <- as_adj(net, sparse=F)
	for(j in 1:nrow(g1b)){
		net.mat[g1b$from[j], g1b$to[j]] <- g1b$raw.weight[j]
		net.mat[g1b$to[j], g1b$from[j]] <- g1b$raw.weight[j]
	}
	new.mat <- net.mat
	new.mat[] <- 0
	cum.mat <- new.mat

	nattempts <- rep(1, ncol(net.mat)) # Remove heterogeneity assumption, limit each individual to only one attempt

	nit <- 5 # time steps
	
	for(k in 1:nit){
		for(j in 1:nrow(new.mat)){
			pot.ps <- which(net.mat[j,]>0) # Stable partner preferences
			if(length(pot.ps)<1){
				pot.ps <- sample((1:ncol(new.mat))[-j], 1)
			}
			if(length(pot.ps)>1){
				pot.ps <- pot.ps[order(net.mat[j,][which(net.mat[j,]>0)], decreasing=T)]
				if(length(pot.ps) < nrow(net.mat)-1){
					pot.ps <- c(pot.ps, sample((1:ncol(new.mat))[!(1:ncol(new.mat)) %in% c(j, pot.ps)], 1))
				}
				new.mat[j, sample(pot.ps, nattempts[j], replace=T, prob=length(pot.ps):1)] <- 1 # Removed requirement for reciprocation
			} else {
				new.mat[j, pot.ps] <- 1 # Removed requirement for reciprocation
			}
		}
		new.mat[upper.tri(new.mat)] <- new.mat[upper.tri(new.mat)] + t(new.mat)[upper.tri(t(new.mat))]
		new.mat[] <- floor(new.mat[])
		new.mat[lower.tri(new.mat)] <- t(new.mat)[lower.tri(t(new.mat))]
		cum.mat <- new.mat + cum.mat
		new.mat[] <- 0
	}

	n.dat <- igraph::as_data_frame(graph.adjacency(cum.mat>0, mode='undirected'))

	# Calculate stability:
	new.dat_pref_only$stability[i] <- length(intersect(unique(c(paste(n.dat$from,n.dat$to), paste(n.dat$to, n.dat$from))), unique(c(paste(g1b$from,g1b$to), paste(g1b$to, g1b$from))))) / length(union(unique(c(paste(n.dat$from,n.dat$to), paste(n.dat$to, n.dat$from))), unique(c(paste(g1b$from,g1b$to), paste(g1b$to, g1b$from)))))

	print(i)
}
summary(new.dat_pref_only)

sumP <- summary(mod <- lm(scale(stability)~ scale(log(nbird)) + scale(network.weight) + scale(density), data=new.dat_pref_only)) 
sumP <- as.data.frame(coef(sumP)); sumP
mod <- lm(stability ~ log(nbird) + network.weight + density, data=new.dat_pref_only) # Unstandarized for plotting

b <- visreg(mod, 'network.weight', plot=F, type='conditional')
c <- visreg(mod, 'density', plot=F, type='conditional')

dev.new(width=4.5, height=2) # Plot not used
par(mfrow=c(1,2), las=1, bty='l', mar=c(3,3,0.2,0.2), mgp=c(1.5,0.25,0), tck=-0.03)

plot(visregRes ~ network.weight, data=b$res, pch=16, col=rgb(0,0,0,0.025), cex=1, ylim=c(0,1), ylab='Network stability', xlab='Network weight', xlim=c(0.4,2.65), type='n')
points(visregFit ~ network.weight, data=b$fit, type='l', lwd=2.5)
mypolyfun(object=b, variable='network.weight')

plot(visregRes ~ density, data=c$res, pch=16, col=rgb(0,0,0,0.025), cex=1, ylim=c(0,1), ylab='', xlab='Network density', xlim=c(0.13,0.82), type='n')
points(visregFit ~ density, data=c$fit, type='l', lwd=2.5)
mypolyfun(object=c, variable='density')




# Null 2b this model assumes heterogeneity ONLY, but removed stable partner preferences and requirement for reciprocation:
######################################################################################################################################

new.dat_activ_only <- data.frame(nbird=rep(11:20, each=15*20), target.density=rep(seq(0.2,0.8, length.out=20), each=15), weight.factor=seq(0.2,2,length.out=15), network.weight=NA, density=NA, stability=NA); dim(new.dat_activ_only)

for(i in 1:nrow(new.dat_activ_only)){

	set.seed(i)
	
	# Create the starting network:
	net <- net_create(gsize=new.dat_activ_only$nbird[i], target.density=new.dat_activ_only$target.density[i])

	# Apply edge weights sampling from manakin data:
	g1b <- igraph::as_data_frame(net)
	g1b$raw.weight <- sample(allweights[allweights>0 & allweights<501], nrow(g1b))
	g1b$raw.weight <- ceiling(new.dat_activ_only$weight.factor[i]*(g1b$raw.weight)) # Apply the weight factor:
	g1b$raw.weight[g1b$raw.weight>500] <- 500 # Truncate at 500
	
	# Get network properties:
	new.dat_activ_only$network.weight[i] <- mean(log(g1b$raw.weight))
	new.dat_activ_only$density[i] <- edge_density(net)
	
	# Run the simulation:	
	net.mat <- as_adj(net, sparse=F)
	for(j in 1:nrow(g1b)){
		net.mat[g1b$from[j], g1b$to[j]] <- g1b$raw.weight[j]
		net.mat[g1b$to[j], g1b$from[j]] <- g1b$raw.weight[j]
	}
	new.mat <- net.mat
	new.mat[] <- 0
	cum.mat <- new.mat

	nattempts <- ceiling(log(rowSums(net.mat)+1, base=10) + 0.1) # Among-individual heterogeneity assumption

	nit <- 5 # time steps
	
	for(k in 1:nit){
		for(j in 1:nrow(new.mat)){
			pot.ps <- sample(1:ncol(net.mat)) # Removed assumption of stable preferences; all individuals have random preferences
			if(length(pot.ps)>1){
				new.mat[j, sample(pot.ps, nattempts[j], replace=T)] <- 1 # Removed requirement for reciprocation
			} else {
				new.mat[j, pot.ps] <- 1 # Removed requirement for reciprocation
			}
		}
		new.mat[upper.tri(new.mat)] <- new.mat[upper.tri(new.mat)] + t(new.mat)[upper.tri(t(new.mat))]
		new.mat[] <- floor(new.mat[])
		new.mat[lower.tri(new.mat)] <- t(new.mat)[lower.tri(t(new.mat))]
		cum.mat <- new.mat + cum.mat
		new.mat[] <- 0
	}

	n.dat <- igraph::as_data_frame(graph.adjacency(cum.mat>0, mode='undirected'))

	# Calculate stability
	new.dat_activ_only$stability[i] <- length(intersect(unique(c(paste(n.dat$from,n.dat$to), paste(n.dat$to, n.dat$from))), unique(c(paste(g1b$from,g1b$to), paste(g1b$to, g1b$from))))) / length(union(unique(c(paste(n.dat$from,n.dat$to), paste(n.dat$to, n.dat$from))), unique(c(paste(g1b$from,g1b$to), paste(g1b$to, g1b$from)))))
	
	print(i)
}
summary(new.dat_activ_only)

sumA <- summary(mod <- lm(scale(stability)~ scale(log(nbird)) + scale(network.weight) + scale(density), data=new.dat_activ_only)) 
sumA <- as.data.frame(coef(sumA)); sumA
mod <- lm(stability ~ log(nbird) + network.weight + density, data=new.dat_activ_only) # Unstandardized for plotting

b <- visreg(mod, 'network.weight', plot=F, type='conditional')
c <- visreg(mod, 'density', plot=F, type='conditional')

dev.new(width=4.5, height=2) # Plots not used
par(mfrow=c(1,2), las=1, bty='l', mar=c(3,3,0.2,0.2), mgp=c(1.5,0.25,0), tck=-0.03)

plot(visregRes ~ network.weight, data=b$res, pch=16, col=rgb(0,0,0,0.025), cex=1, ylim=c(0,1), ylab='Network stability', xlab='Network strength', xlim=c(0.4,2.65), type='n')
points(visregFit ~ network.weight, data=b$fit, type='l', lwd=2.5)
mypolyfun(object=b, variable='network.weight')

plot(visregRes ~ density, data=c$res, pch=16, col=rgb(0,0,0,0.025), cex=1, ylim=c(0,1), ylab='', xlab='Network density', xlim=c(0.13,0.82), type='n')
points(visregFit ~ density, data=c$fit, type='l', lwd=2.5)
mypolyfun(object=c, variable='density')




# Null 2c this assumes the requirement for reciprocation, but removes assumnption of stable partner preferences and among-individual heterogeneity:
###################################################################################################################################################

new.dat_recip_only <- data.frame(nbird=rep(11:20, each=15*20), target.density=rep(seq(0.2,0.8, length.out=20), each=15), weight.factor=seq(0.2,2,length.out=15), network.weight=NA, density=NA, stability=NA); dim(new.dat_recip_only)

for(i in 1:nrow(new.dat_recip_only)){

	set.seed(i)

	# Create the starting network:
	net <- net_create(gsize=new.dat_recip_only$nbird[i], target.density=new.dat_recip_only$target.density[i])
	
	# Apply edge weights sampling from manakin data:
	g1b <- igraph::as_data_frame(net)
	g1b$raw.weight <- sample(allweights[allweights>0 & allweights<501], nrow(g1b))
	g1b$raw.weight <- ceiling(new.dat_recip_only$weight.factor[i]*(g1b$raw.weight)) # Apply the weight factor:
	g1b$raw.weight[g1b$raw.weight>500] <- 500 # Truncate at 500
	
	# Get network properties:
	new.dat_recip_only$network.weight[i] <- mean(log(g1b$raw.weight))
	new.dat_recip_only$density[i] <- edge_density(net)

	# Run the simulation:
	net.mat <- as_adj(net, sparse=F)
	for(j in 1:nrow(g1b)){
		net.mat[g1b$from[j], g1b$to[j]] <- g1b$raw.weight[j]
		net.mat[g1b$to[j], g1b$from[j]] <- g1b$raw.weight[j]
	}
	new.mat <- net.mat
	new.mat[] <- 0
	cum.mat <- new.mat

	nattempts <- rep(1, ncol(net.mat)) # Remove heterogeneity assumption, limit each individual to only one attempt

	nit <- 5	 # time steps

	for(k in 1:nit){
		for(j in 1:nrow(new.mat)){
			pot.ps <- sample(1:ncol(net.mat)) # Removed assumption of stable preferences; all individuals have random preferences
			if(length(pot.ps)>1){
				new.mat[j, sample(pot.ps, nattempts[j], replace=T)] <- 0.5
			} else {
				new.mat[j, pot.ps] <- 0.5
			}
		}
		new.mat[upper.tri(new.mat)] <- new.mat[upper.tri(new.mat)] + t(new.mat)[upper.tri(t(new.mat))]
		new.mat[] <- floor(new.mat[])
		new.mat[lower.tri(new.mat)] <- t(new.mat)[lower.tri(t(new.mat))]
		cum.mat <- new.mat + cum.mat
		new.mat[] <- 0
	}

	n.dat <- igraph::as_data_frame(graph.adjacency(cum.mat>0, mode='undirected'))

	# Calculate stability
	new.dat_recip_only$stability[i] <- length(intersect(unique(c(paste(n.dat$from,n.dat$to), paste(n.dat$to, n.dat$from))), unique(c(paste(g1b$from,g1b$to), paste(g1b$to, g1b$from))))) / length(union(unique(c(paste(n.dat$from,n.dat$to), paste(n.dat$to, n.dat$from))), unique(c(paste(g1b$from,g1b$to), paste(g1b$to, g1b$from)))))

	print(i)
}
summary(new.dat_recip_only)

sumR <- summary(mod <- lm(scale(stability)~ scale(log(nbird)) + scale(network.weight) + scale(density), data=new.dat_recip_only)) 
sumR <- as.data.frame(coef(sumR)); sumR
mod <- lm(stability ~ log(nbird) + network.weight + density, data=new.dat_recip_only) # Unstandardized for plotting

b <- visreg(mod, 'network.weight', plot=F, type='conditional')
c <- visreg(mod, 'density', plot=F, type='conditional')

dev.new(width=4.5, height=2) # Plot not used
par(mfrow=c(1,2), las=1, bty='l', mar=c(3,3,0.2,0.2), mgp=c(1.5,0.25,0), tck=-0.03)

plot(visregRes ~ network.weight, data=b$res, pch=16, col=rgb(0,0,0,0.025), cex=1, ylim=c(0,1), ylab='Network stability', xlab='Network weight', xlim=c(0.4,2.65), type='n')
points(visregFit ~ network.weight, data=b$fit, type='l', lwd=2.5)
mypolyfun(object=b, variable='network.weight')

plot(visregRes ~ density, data=c$res, pch=16, col=rgb(0,0,0,0.025), cex=1, ylim=c(0,1), ylab='', xlab='Network density', xlim=c(0.13,0.82), type='n')
points(visregFit ~ density, data=c$fit, type='l', lwd=2.5)
mypolyfun(object=c, variable='density')




# Compile results in Table:
(all_sim_results <- data.frame(model=rep(c('NULL1_Rewire','NULL2_IBM','recip_only','pref_only','heterogen_only','manakin_data','full_IBM'), each=3), coef=c('size', 'weight', 'density'), est=NA, lower=NA, upper=NA, p=NA))

all_sim_results[all_sim_results$model=='NULL1_Rewire',]$est <- sumN$Estimate[2:4]
all_sim_results[all_sim_results$model=='NULL1_Rewire',]$lower <- sumN$Estimate[2:4] - 1.96*sumN$Std[2:4]
all_sim_results[all_sim_results$model=='NULL1_Rewire',]$upper <- sumN$Estimate[2:4] + 1.96*sumN$Std[2:4]
all_sim_results[all_sim_results$model=='NULL1_Rewire',]$p <- sumN$Pr[2:4] 

all_sim_results[all_sim_results$model=='NULL2_IBM',]$est <- sumN2$Estimate[2:4]
all_sim_results[all_sim_results$model=='NULL2_IBM',]$lower <- sumN2$Estimate[2:4] - 1.96*sumN2$Std[2:4]
all_sim_results[all_sim_results$model=='NULL2_IBM',]$upper <- sumN2$Estimate[2:4] + 1.96*sumN2$Std[2:4]
all_sim_results[all_sim_results$model=='NULL2_IBM',]$p <- sumN2$Pr[2:4] 

all_sim_results[all_sim_results$model=='pref_only',]$est <- sumP$Estimate[2:4]
all_sim_results[all_sim_results$model=='pref_only',]$lower <- sumP$Estimate[2:4] - 1.96* sumP$Std[2:4]
all_sim_results[all_sim_results$model=='pref_only',]$upper <- sumP$Estimate[2:4] + 1.96* sumP$Std[2:4]
all_sim_results[all_sim_results$model=='pref_only',]$p <- sumP$Pr[2:4] 

all_sim_results[all_sim_results$model=='heterogen_only',]$est <- sumA$Estimate[2:4]
all_sim_results[all_sim_results$model=='heterogen_only',]$lower <- sumA$Estimate[2:4] - 1.96* sumA$Std[2:4]
all_sim_results[all_sim_results$model=='heterogen_only',]$upper <- sumA$Estimate[2:4] + 1.96* sumA$Std[2:4]
all_sim_results[all_sim_results$model=='heterogen_only',]$p <- sumA$Pr[2:4] 

all_sim_results[all_sim_results$model=='recip_only',]$est <- sumR$Estimate[2:4]
all_sim_results[all_sim_results$model=='recip_only',]$lower <- sumR$Estimate[2:4] - 1.96* sumR$Std[2:4]
all_sim_results[all_sim_results$model=='recip_only',]$upper <- sumR$Estimate[2:4] + 1.96* sumR$Std[2:4]
all_sim_results[all_sim_results$model=='recip_only',]$p <- sumR$Pr[2:4] 

# Get the manakin data fit & main individual-based model fit, to plot standardized coefs for figure 2:
load('manakin_best_fit_regression_model.RData'); sumD <- as.data.frame(coef(summary(mod)))
all_sim_results[all_sim_results$model=='manakin_data',]$est <- sumD$Estimate[5:7]
all_sim_results[all_sim_results$model=='manakin_data',]$lower <- sumD$Estimate[5:7] - 1.96* sumD$Std[5:7]
all_sim_results[all_sim_results$model=='manakin_data',]$upper <- sumD$Estimate[5:7] + 1.96* sumD$Std[5:7]
all_sim_results[all_sim_results$model=='manakin_data',]$p <- sumD$Pr[5:7] 

load('individual_based_simulation_fitted_regression_model.RData') ; sumIBM <- as.data.frame(coef(summary(mod)))
all_sim_results[all_sim_results$model=='full_IBM',]$est <- sumIBM$Estimate[2:4]
all_sim_results[all_sim_results$model=='full_IBM',]$lower <- sumIBM$Estimate[2:4] - 1.96* sumIBM$Std[2:4]
all_sim_results[all_sim_results$model=='full_IBM',]$upper <- sumIBM$Estimate[2:4] + 1.96* sumIBM$Std[2:4]
all_sim_results[all_sim_results$model=='full_IBM',]$p <- sumIBM$Pr[2:4] 

all_sim_results


plot_results <- subset(all_sim_results, model %in% c('manakin_data','full_IBM','NULL2_IBM'))
plot_results <- plot_results[c(1:3,7:9,4:6),]

dev.new(width=4, height=5)
par(las=1, bty='l', mfrow=c(1,3), mar=c(4,4,2,0.1), mgp=c(2.5,0.25,0), tck=-0.02, cex.main=0.75)
plot(c(1:3) ~ est, data=subset(plot_results, coef=='size'), ylab='', xlab='Std. coef', yaxt='n', ylim=c(1,4), pch=16, type='p', cex=0.5, main='Network size effect', xlim=c(-0.9,0.2), xaxt='n'); abline(v=0, lty=3, lwd=0.2)
axis(1, at=c(-0.8,-0.4,0), cex.axis=0.75)
segments(y0=c(1:3), y1=c(1:3), x0=subset(plot_results, coef=='size')$lower, x1=subset(plot_results, coef=='size')$upper, lwd=0.45)
polygon(y=c(0,4,4,0), x=c(-0.654165700, -0.654165700, 0.07966928, 0.07966928), border=NA, col=rgb(0,0,0,0.05))
axis(2, at=1:3, labels=unique(subset(plot_results, coef=='size')$model), cex.axis=0.5)

plot(c(1:3) ~ est, data=subset(plot_results, coef=='weight'), ylab='', xlab='Std. coef', yaxt='n', ylim=c(1,4), pch=16, type='p', cex=0.5, main='Network weight effect', xlim=c(-0.1,0.8)); abline(v=0, lty=3, lwd=0.2)
segments(y0=c(1:3), y1=c(1:3), x0=subset(plot_results, coef=='weight')$lower, x1=subset(plot_results, coef=='weight')$upper, lwd=0.45)
polygon(y=c(0,7,7,0), x=c(0.153688093, 0.153688093, 0.63343508, 0.63343508), border=NA, col=rgb(0,0,0,0.05))

plot(c(1:3) ~ est, data=subset(plot_results, coef=='density'), ylab='', xlab='Std. coef', yaxt='n', ylim=c(1,4), pch=16, type='p', cex=0.5, main='Density effect', xlim=c(-1.05, 1.05)); abline(v=0, lty=3, lwd=0.2)
segments(y0=c(1:3), y1=c(1:3), x0=subset(plot_results, coef=='density')$lower, x1=subset(plot_results, coef=='density')$upper, lwd=0.45)
polygon(y=c(0,7,7,0), x=c(-0.980600698, -0.980600698, -0.27247434, -0.27247434), border=NA, col=rgb(0,0,0,0.05))


