
# Reciprocity and behavioral heterogeneity govern the stability of social networks 
# Roslyn Dakin and T. Brandt Ryder

# This script performs the analysis of the persistence of specific manakin social partnerships, annually
# Figure 3, Figure 4a

library(igraph)
library(dplyr)
library(brms)
library(lme4)
library(lmerTest)
library(visreg)

# Get social partnership data:
dyads2 <- read.csv('dyads_annual_tracked_next_season.csv', stringsAsFactors=F); dim(dyads2) # 669 unique partnerships, wherein both individuals were also tagged/studied the following year 

# Get social interaction data:
year3 <- read.csv('filtered_interactions_nodups_times_durations_052418.csv', stringsAsFactors=F)[,c('date','nTime','Color1','Color2','Lek','studyyear','DL','sitefix')]; dim(year3)
year12 <- read.csv('filtered_interactions_nodups_times_durations_011018.csv', stringsAsFactors=F)[,c('date','nTime','Color1','Color2','Lek','studyyear','DL','sitefix')]; dim(year12)
ally <- rbind(year12, year3); head(ally); dim(ally) # Duplicate and reverse individual 1 & individual 2 to account for all interactions:
allyX <- ally
allyX$Color1 <- ally$Color2; allyX$Color2 <- ally$Color1
dim(ally); dim(allyX)
ally <- rbind(ally, allyX)
ally$Color1[ally$Color1=='OX-'] <- 'OX' # Correct ID error
ally$Color2[ally$Color2=='OX-'] <- 'OX'
ally$sitefix[ally$sitefix=='OX-'] <- 'OX'
all(levels(factor(ally$Color1))==levels(factor(ally$Color2))); dim(ally) # 73,770

# Get year-specific status:
meta <- unique(read.csv('bird_metadata_2015-2018.csv')[,c('ID','year','Status')]); dim(meta)
meta$ID_year <- paste(meta$ID, meta$year, sep='_')
dyads2$ColorA_year <- paste(dyads2$ColorA,dyads2$studyyear,sep='_'); dyads2$ColorB_year <- paste(dyads2$ColorB,dyads2$studyyear,sep='_')
dyads2 <- merge(dyads2, meta[,c('ID_year','Status')], by.x='ColorA_year', by.y='ID_year'); dim(dyads2)
dyads2 <- merge(dyads2, meta[,c('ID_year','Status')], by.x='ColorB_year', by.y='ID_year'); dim(dyads2)

dyads2$DSTatus <- paste(dyads2$Status.x, dyads2$Status.y, sep='_')
dyads2$DSTatus <- ifelse(dyads2$DSTatus=='terr_floa'|dyads2$DSTatus=='floa_terr', 'floa_terr', dyads2$DSTatus); table(dyads2$DSTatus)

# Get continous distance metric of space-use overlap:
space <- read.csv('dyads_plus_spatial_overlap_metrics.csv', stringsAsFactors=F)[,-1]; head(space); dim(space) 
length(levels(factor(space$dyad))); length(levels(factor(paste(space$dyad, space$studyyear)))) # 1,836 partnership-years, 1,530 partnerships
space$dyad_year <- paste(space$dyad, space$studyyear, sep='_')
dyads2$dyad_year <- paste(dyads2$dyad, dyads2$studyyear, sep='_')
dyads2 <- merge(dyads2, space[,c('dist.terrCHI','dyad_year')]); dim(dyads2)

# Build annual networks to determine properties of partnerships as networks edges (edge weight, edge betweenness, edge connectivity), and to summarize edge weights for later use in simulations:
g1 <- unique(subset(ally, studyyear=='15-16')[,c('Color1','Color2')])
temp <- summarize(group_by(subset(ally, studyyear=='15-16'), Color1, Color2), n=n())
g1$weight <- temp$n[match(paste(g1$Color1, g1$Color2), paste(temp$Color1, temp$Color2))]; g1 <- subset(g1, Color1 < Color2)
g1w <- graph.data.frame(g1, directed=F)

g2 <- unique(subset(ally, studyyear=='16-17')[,c('Color1','Color2')])
temp <- summarize(group_by(subset(ally, studyyear=='16-17'), Color1, Color2), n=n())
g2$weight <- temp$n[match(paste(g2$Color1, g2$Color2), paste(temp$Color1, temp$Color2))]; g2 <- subset(g2, Color1 < Color2)
g2w <- graph.data.frame(g2, directed=F)

g3 <- unique(subset(ally, studyyear=='17-18')[,c('Color1','Color2')])
temp <- summarize(group_by(subset(ally, studyyear=='17-18'), Color1, Color2), n=n())
g3$weight <- temp$n[match(paste(g3$Color1, g3$Color2), paste(temp$Color1, temp$Color2))]; g3 <- subset(g3, Color1 < Color2) 
g3w <- graph.data.frame(g3, directed=F)

# Setup network.met dataframe to store properties:
network.met <- data.frame(dyad_year=dyads2$dyad_year, true.weight=rep(NA, length=dim(dyads2)[1]), ebw=NA, connect=NA)
network.met$true.weight[dyads2$studyyear=='15-16'] <- c(edge_attr(g1w)$weight, edge_attr(g1w)$weight)[match(dyads2$dyad[dyads2$studyyear=='15-16'], c(paste(as_edgelist(g1w)[,1], as_edgelist(g1w)[,2], sep='_'), paste(as_edgelist(g1w)[,2], as_edgelist(g1w)[,1], sep='_')))]
network.met$true.weight[dyads2$studyyear=='16-17'] <- c(edge_attr(g2w)$weight, edge_attr(g2w)$weight)[match(dyads2$dyad[dyads2$studyyear=='16-17'], c(paste(as_edgelist(g2w)[,1], as_edgelist(g2w)[,2], sep='_'), paste(as_edgelist(g2w)[,2], as_edgelist(g2w)[,1], sep='_')))]
network.met$ebw[dyads2$studyyear=='15-16'] <- c(edge_betweenness(g1w), edge_betweenness(g1w))[match(dyads2$dyad[dyads2$studyyear=='15-16'], c(paste(as_edgelist(g1w)[,1], as_edgelist(g1w)[,2], sep='_'), paste(as_edgelist(g1w)[,2], as_edgelist(g1w)[,1], sep='_')))]
network.met$ebw[dyads2$studyyear=='16-17'] <- c(edge_betweenness(g2w), edge_betweenness(g2w))[match(dyads2$dyad[dyads2$studyyear=='16-17'], c(paste(as_edgelist(g2w)[,1], as_edgelist(g2w)[,2], sep='_'), paste(as_edgelist(g2w)[,2], as_edgelist(g2w)[,1], sep='_')))]
summary(network.met)
plot(log(ebw+1) ~ jitter(log(true.weight),50), network.met, pch=16, col=as.factor(dyads2$studyyear)) # The strongest ties have lower edge betweenness 

# Now get edge connectivity for each edge:
for(i in 1:dim(network.met)[1]){
	studyyearx <- dyads2$studyyear[i]
	b1 <- dyads2$ColorA[i]
	b2 <- dyads2$ColorB[i]
	v1 <- paste(b1, b2, sep='|')
	v2 <- paste(b2, b1, sep='|')	
	if(studyyearx=='15-16'){
		network.met$connect[i] <- edge_connectivity(g1w, b1, b2)
	}
	if(studyyearx=='16-17'){
		network.met$connect[i] <- edge_connectivity(g2w, b1, b2)
	}
	print(i)	
}
par(mfrow=c(2,2), las=1, bty='l')
plot(ebw ~ connect, network.met); plot(true.weight ~ connect, network.met)

dyad.an <- merge(dyads2, network.met, by='dyad_year', all.x=T, all.y=F); dim(dyad.an); head(dyad.an)

# Determine where did they primarily interact, to account for potential (environmental) variance among Leks:
dyad.an$Lek <- NA; dyad.an$Lek_2 <- NA
for(i in 1:nrow(dyad.an)){
	yr <- dyad.an$studyyear[i]
	b1 <- dyad.an$ColorA[i]
	b2 <- dyad.an$ColorB[i]
	dyad.an$Lek[i] <- names(sort(table(subset(ally, studyyear==yr & Color1==b1 & Color2==b2)$Lek), decreasing=T))[1]
	yr <- dyad.an$nextyear[i]
	dyad.an$Lek_2[i] <- names(sort(table(subset(ally, studyyear==yr & Color1==b1 | Color2==b2)$Lek), decreasing=T))[1] # have to use "or" because the relationshp may not have occurred year 2	
}

# Now add the permutation filter:
perm <- read.csv('dyads_annual_plus_perm.csv', stringsAsFactors=F); dim(perm); head(perm) # 1836 annual partnerships
perm$stab <- NA; perm$nsesh <- NA; perm$nposs <- NA
perm$new.real <- perm$edge.weight > perm$edge.weightPERM & perm$edge.weight >= 6; table(perm$new.real) # most are not 'real' by this stringent criteria
perm$dyad_year <- paste(perm$dyad, perm$studyyear, sep='_'); head(perm)

dyad.an$new.real2 <- NA # checks, was the partnership significant the following year?
for(i in 1:nrow(dyad.an)){
	res <- subset(perm, dyad_year==paste(dyad.an$dyad[i],	dyad.an$nextyear[i], sep='_'))$new.real
	if(length(res)>0){
		dyad.an$new.real2[i] <- res
	} else {
		dyad.an$new.real2[i] <- FALSE
	}
}

dyad.an$spatial.overlapL <- log(1/(dyad.an$dist.terrCHI+1)) # more positive value = greater overlap in space use
dyad.an$edge.weightL <- log(dyad.an$true.weight)
dyad.an$ebwL <- log(dyad.an$ebw+1)
dyad.an$connectL <- log(dyad.an$connect)
dyad.an[dyad.an$ColorA=='BX',c('ColorA','studyyear')]$ColorA <- 'BXa' # Correct an error

# Add sampling effort:
stime <- read.csv('stime_by_date.csv', stringsAsFactors=F); head(stime); dim(stime)
dyad.an$samp_yr_1 <- NA; dyad.an$samp_yr_2 <- NA; head(dyad.an)
stime_yr <- summarize(group_by(stime, Lek, Terr, studyyear), sampTime=sum(sampTime))
stime_yr <- subset(stime_yr, sampTime>20); hist(stime_yr$sampTime, breaks=30)
for(i in 1:nrow(dyad.an)){
	dyad.an$samp_yr_1[i] <- mean(subset(stime_yr, studyyear==dyad.an$studyyear[i] & Lek==dyad.an$Lek[i])$sampTime)
	dyad.an$samp_yr_2[i] <- mean(subset(stime_yr, studyyear==dyad.an$nextyear[i] & Lek==dyad.an$Lek_2[i])$sampTime)
	print(i)
}


# Fit the binomial MMM (multiple memberhip model):

dyad.an$trials <- 1
fit_mm_23 <- brm(as.numeric(new.real2)|trials(trials) ~ studyyear + edge.weightL + ebwL + connectL + spatial.overlapL + DSTatus + samp_yr_1 + samp_yr_2 + (1|Lek) + (1|mm(ColorA, ColorB)), data=dyad.an, chains=4, iter=2000, control=list(adapt_delta=0.98, max_treedepth=10), family=binomial); summary(fit_mm_23)
bayes_R2(fit_mm_23) # and 36% of var explained in persistence of partnerships to the following year.
fit_mm_23std <- brm(as.numeric(new.real2)|trials(trials) ~ studyyear + scale(edge.weightL) + scale(ebwL) + scale(connectL) + scale(spatial.overlapL) + DSTatus + samp_yr_1 + samp_yr_2 + (1|Lek) + (1|mm(ColorA, ColorB)), data=dyad.an, chains=4, iter=2000, control=list(adapt_delta=0.98, max_treedepth=10), family=binomial); summary(fit_mm_23std) # Standardized estimates
# Strong, positive effects of edge weight and spatial overlap on persistence, and a strong negative effect of edge connectivity on persistence.
fit_mm_23NULL <- brm(as.numeric(new.real2)|trials(trials) ~ studyyear + DSTatus + samp_yr_1 + samp_yr_2 + (1|Lek) + (1|mm(ColorA, ColorB)), data=dyad.an, chains=4, iter=2000, control=list(adapt_delta=0.999, max_treedepth=10), family=binomial) # This model excludes the network info to extract predicted effect of individuals on partnership persistence
# save(fit_mm_23, fit_mm_23std, fit_mm_23NULL, dyad.an, file='fitted_mmm_model_effort.RData')

# Fit a simple, GLMM, to get predictions based on edge properties. This gets used later to parameterize an edge-level simulation in the simulation analyses:

fit_mm_23L <- lme4::glmer(as.numeric(new.real2) ~ studyyear + edge.weightL + ebwL + connectL + spatial.overlapL + DSTatus + (1|Lek), data=subset(dyad.an), family=binomial); summary(fit_mm_23L) # Consistent with the MMM analysis above, just doesn't account for effect of individual ID

pred.data <- cbind(predict(fit_mm_23L, newdata=expand.grid(connectL=log(seq(1,100,length.out=100)), edge.weightL=log(seq(1,500,length.out=500)), Lek='H3-1000', studyyear=c('15-16','16-17'), ebwL=1.6, spatial.overlapL=-9.2, DSTatus='floa_terr'), type='response'), expand.grid(connectL=log(seq(1,100,length.out=100)), edge.weightL=log(seq(1,500,length.out=500)), Lek='H3-1000', studyyear=c('15-16','16-17'), ebwL=1.6, spatial.overlapL=-9.2, DSTatus='floa_terr')); head(pred.data); dim(pred.data)
names(pred.data)[1] <- 'pred'
pred.data <- summarize(group_by(pred.data, Lek, edge.weightL, connectL), pred=mean(pred))
pred.data$connect <- round(exp(pred.data$connectL)); pred.data$edge.weight <- round(exp(pred.data$edge.weightL))
par(mfrow=c(2,2), las=1, bty='l')
plot(pred ~ connect, pred.data, type='p', pch=16, col=rgb(0,0,0,0.05), cex=0.2) 
plot(pred ~ edge.weight, pred.data, type='p', pch=16, col=rgb(0,0,0,0.05), cex=0.2) 
allweights <- sort(c(edge_attr(g1w)$weight, edge_attr(g2w)$weight, edge_attr(g3w)$weight))
pred.data <- pred.data[,-1]
# save(pred.data, allweights, file='edge_level_predictions.RData')

# Plot partnership persistence vs. edge weight and connectivity:
# Function to plot predictions of the fitted MMM model:
myplotfun <- function(mod, var='edge.weightL', mn=0, mx=6.2, bb=0.01){
	if(var=='DSTatus'){
		pre.data <- expand.grid(studyyear=c('15-16','16-17'), DSTatus=c('floa_floa','floa_terr','terr_terr'), connectL=2.7, spatial.overlapL=-9.7, ebwL=1.65, edge.weightL=1.5, samp_yr_1=252, samp_yr_2=273, ColorA='PMPX', ColorB='MXYO', Lek='H3-1000', trials=1)
		pred.data <- data.frame(predict(fit_mm_23, newdata=pre.data))
		pred.data <- cbind(pred.data[,1], pre.data)
		names(pred.data)[1] <- 'Estimate'
		pred.data <- summarize(group_by(pred.data, DSTatus), Estimate=mean(Estimate))
		other.pred <- data.frame(predict(fit_mm_23))
		return(list(other.pred, pred.data))

	} else {
		replace <- seq(mn, mx, by=bb)
		pre.data <- expand.grid(studyyear=c('15-16','16-17'), DSTatus=c('floa_floa','floa_terr','terr_terr'), connectL=2.7, spatial.overlapL=-9.7, ebwL=1.65, edge.weightL=1.5, samp_yr_1=252, samp_yr_2=273, ColorA='PMPX', ColorB='MXYO', Lek='H3-1000', var=replace, trials=1)
		pre.data[,var] <- pre.data[,'var']
		pred.data <- data.frame(predict(fit_mm_23, newdata=pre.data))
		pred.data <- cbind(pred.data[,1], pre.data)
		names(pred.data)[1] <- 'Estimate'
		pred.data <- summarize(group_by(pred.data, var), Estimate=mean(Estimate))
		names(pred.data)[1] <- var
		other.pred <- data.frame(predict(fit_mm_23))
		return(list(other.pred, pred.data))
	}	
}

# Plot posterior model predictions for Figure 3:

dev.new(width=4.5, height=2) 
par(mfrow=c(1,2), las=1, bty='l', mar=c(3,3,0.2,0.2), mgp=c(1.5,0.25,0), tck=-0.03)

pred.data <- myplotfun(fit_mm_23, var='edge.weightL', mn=0, mx=6.2, bb=0.01)
plot(dyad.an$new.real2 ~ dyad.an$edge.weightL, ylim=c(-0.01,1.01), xlim=c(0,6.22), type='n', ylab='Partnership persistence', xlab='Edge weight (log)', yaxt='n')
axis(2, at=c(0,0.2,0.4,0.6,0.8,1))
points(Estimate ~ edge.weightL, data=pred.data[[2]], type='l', lwd=2.5)
points(pred.data[[1]]$Estimate ~ dyad.an$edge.weightL, pch=16, cex=0.6, col=rgb(0,0,0,0.05))

pred.data <- myplotfun(fit_mm_23, var='connectL', mn=1.10, mx=3.64, bb=0.01) 
plot(dyad.an$new.real2 ~ dyad.an$connectL, ylim=c(-0.01,1.01), xlim=c(1.10, 3.64), type='n', ylab='Partnership persistence', xlab='Edge connectivity (log)', yaxt='n')
axis(2, at=c(0,0.2,0.4,0.6,0.8,1))
points(Estimate ~ connectL, data=pred.data[[2]], type='l', lwd=2.5)
points(pred.data[[1]]$Estimate ~ dyad.an$connectL, pch=16, cex=0.6, col=rgb(0,0,0,0.05))




# Get blups (posterior individual estimates) from fit_mm_23NULL to examine among-individual variation in annual persistence.
# This is subsequently analyzed in the network level script.

# Function to extract individual estimates:
postcov <- function(mod){
	iter <- mod$fit@sim$iter
	chains <- length(mod$fit@sim$samples)
	fnames <- names(mod$fit@sim$samples[[1]])
	ego <- fnames[grep('Color', fnames)][grep('r_', fnames[grep('Color', fnames)])]
	posterior.ego <- data.frame(matrix(NA, nrow=iter*chains, ncol=length(ego)))
	names(posterior.ego) <- ego
	for(i in 1:chains){
		for(j in 1:length(ego)){
			posterior.ego[((i-1)*iter+1):(i*iter), j] <- (mod$fit@sim$samples[[i]][ego[j]])
		}
	}
	names(posterior.ego) <- gsub(',Intercept', '', names(posterior.ego))
	names(posterior.ego) <- gsub('r_mmColorAColorB', '', names(posterior.ego))
	names(posterior.ego) <- gsub('[[]', '', names(posterior.ego))
	names(posterior.ego) <- gsub('[]]', '', names(posterior.ego))
	ego.med <- apply(posterior.ego, 2, 'median')
	ego.sd <- apply(posterior.ego, 2, 'sd')
	d1 <- data.frame(ID=names(ego.med), ego.med=ego.med, ego.sd=ego.sd)
	return(d1)
}

# Get the estimate of each individual's effect on long-term partnership persistence:
persistence <- postcov(fit_mm_23NULL); head(persistence); dim(persistence)

# Now analyze persistence in relation to individuals' behaviors.
# Get daily behavioral data, and summarize per year:
pdata <- read.csv('daily_social_phenotypes_4.csv', stringsAsFactors=F); dim(pdata); head(pdata)
pdata[pdata$ID1=='BX',]$ID1 <- c('BXa', 'BXa', 'BXb') # Correct error
# Annual average behavioral phentoype, for the unmanipulated individuals:
results.summ <- summarize(group_by(subset(pdata, Tmanip %in% c('none','pre')), ID1), effort=mean(log(pings)), strength=mean(log(num.int+1)), degree=mean(log(num.part+1)), imp=mean(mean.stab), Status=names(sort(table(Status), decreasing=T))[1])
names(results.summ)[1] <- 'Color'
results.summ; dim(results.summ) # 181 individual-years

# Merge the behavioral data with each individual's effect on persistence:
bird.an <- merge(results.summ, persistence, by.x='Color', by.y='ID', all.x=T, all.y=T); dim(bird.an); head(bird.an)

prim.lek <- read.csv('primary_lek_by_pings.csv', stringsAsFactors=F)[,-1]; head(prim.lek); dim(prim.lek)
prim.lek$Color[prim.lek$Color=='BX'] <- c('BXa','BXb')

bird.an$samp <- NA
for(i in 1:nrow(bird.an)){	
	yrs <- unique(subset(prim.lek, Color==bird.an$Color[i])[,c('studyyear')])
	samp <- rep(NA, length(yrs))
	for(j in 1:length(samp)){
		plek <- subset(prim.lek, studyyear==yrs[j] & Color==bird.an$Color[i])$primary.lek
		samp[j] <- mean(subset(stime_yr, Lek==plek & studyyear==yrs[j])$sampTime)	
	}
	bird.an$samp[i] <- mean(samp)
}


# Fit candidate models:
mod.bird1 <- lm(ego.med ~ Status + scale(effort) + scale(samp), data=bird.an)
mod.bird2 <- lm(ego.med ~ Status + scale(strength) + scale(samp), data=bird.an)
mod.bird3 <- lm(ego.med ~ Status + scale(degree) + scale(samp), data=bird.an)
mod.bird4 <- lm(ego.med ~ Status + scale(imp) + scale(samp), data=bird.an)
mod.bird5 <- lm(ego.med ~ Status + scale(effort) + scale(strength) + scale(degree) + scale(imp) + scale(samp), data=bird.an)
mod.bird0 <- lm(ego.med ~ Status + scale(samp), data=bird.an)

# Model comparison shows we have two best-fit models, such taht either strength or degree can explain significant variation in persistence:
modsel <- data.frame(AIC(mod.bird0, mod.bird1, mod.bird2, mod.bird3, mod.bird4, mod.bird5))
modsel <- modsel[with(modsel, order(AIC)),]; modsel$delta <- modsel$AIC - min(modsel$AIC); modsel

summary(mod.bird2) # Each has a significant positive effect with R2 between 10-11%, p < 0.003, n = 91 individuals
summary(mod.bird3)

dev.new(width=3, height=3) # Plot the positive relationship with degree for figure 4a:
par(las=1, bty='l', mar=c(3,3,0.1,0.1), mgp=c(2,0.5,0)) 
visreg(mod.bird3, 'degree', xlab='Average daily degree', ylab='Effect on partnershp persistence')
abline(h=0, lty=3)

