
# Reciprocity and behavioral heterogeneity govern the stability of social networks 
# Roslyn Dakin and T. Brandt Ryder

# This script performs the repeated-measures analysis of individuals within-seasons
# Figure 4b, 4c

library(dplyr)
library(lme4)
library(lmerTest)
library(visreg)

# Our network- and edge-level results indicated that connectivity had a negative impact on social stability, and yet the individuals with the most stable long-term relationships were also the most connected. To resolve this apparent paradox, here we look at repeated measures of individuals within-seasons, to test for a potential trade-off between relationship quantity and quality. Our hypothesis is that this trade-off may be felt within-individuals and yet it can be masked by a positive association between quantity and quality at the among-individual level.

# Get social interaction data:
year3 <- read.csv('filtered_interactions_nodups_times_durations_052418.csv', stringsAsFactors=F)[,c('date','nTime','Color1','Color2','Lek','studyyear','DL','sitefix')]; dim(year3)
year12 <- read.csv('filtered_interactions_nodups_times_durations_011018.csv', stringsAsFactors=F)[,c('date','nTime','Color1','Color2','Lek','studyyear','DL','sitefix')]; dim(year12)
ally <- rbind(year12, year3); head(ally); dim(ally) # Duplicate and reverse individual 1 & individual 2 to account for all interactions:
allyX <- ally
allyX$Color1 <- ally$Color2; allyX$Color2 <- ally$Color1
dim(ally); dim(allyX)
ally <- rbind(ally, allyX)
ally$Color1[ally$Color1=='OX-'] <- 'OX' # Correct ID error
ally$Color2[ally$Color2=='OX-'] <- 'OX'
ally$sitefix[ally$sitefix=='OX-'] <- 'OX'
all(levels(factor(ally$Color1))==levels(factor(ally$Color2))); dim(ally) # 73,770

# Set up data frame of repeataed measures of individuals per recording sessions:
bird.dat <- data.frame(unique(ally[,c('Color1','studyyear','DL')]), n.partnerships=NA, prop.persisting=NA, Status=NA, sample_eff=NA); dim(bird.dat) # 882

# Get recording effort per DL-bird repeated measures:
stime <- read.csv('stime_by_date.csv', stringsAsFactors=F)[,-1]; head(stime); dim(stime)
for(i in 1:nrow(bird.dat)){
	PL <- names(sort(table(subset(ally, Color1==bird.dat$Color1[i] & studyyear==bird.dat$studyyear[i] & DL==bird.dat$DL[i])$Lek), decreasing=T))[1]
	ldat <- subset(stime, studyyear==bird.dat$studyyear[i] & DL==bird.dat$DL[i] & Lek==PL)
	ldat <- summarize(group_by(ldat, Terr), sampTime=sum(sampTime))
	bird.dat$sample_eff[i] <- mean(ldat$sampTime)
	print(i)	
}

# Quantify the number of partnerships per recording session and persistence of those partnerships to the next session within that season, if it exists:
for(i in 1:nrow(bird.dat)){ #
	print(i)	

	yr <- bird.dat$studyyear[i]
	ID <- bird.dat$Color1[i]
	dl <- bird.dat$DL[i]
	
	temp1 <- subset(ally, Color1==ID & studyyear==yr & DL==dl)
	temp2 <- subset(ally, Color1==ID & studyyear==yr & DL==dl+1)
	
	bird.dat$Status[i] <- ifelse(any(temp1$sitefix==ID), 'terr', 'floa')			
	pot.p <- unique(subset(ally, studyyear==yr & DL==dl & Lek %in% temp1$Lek)$Color1)
	bird.dat$n.partnerships[i] <- length(table(temp1$Color2))

	# Persistence of partnerships:	
	if(nrow(temp2)>0){
		bird.dat$prop.persisting[i] <- sum(names(table(temp1$Color2)) %in% names(table(temp2$Color2))) / length(table(temp1$Color2)) 		
	}
	print(i)
}

head(bird.dat); tail(bird.dat)
summary(bird.dat[,c(4:5)]); sum(!is.na(bird.dat$n.persisting)) # n = 565 observations for analysis

# Within-individual centering to get a relative measure of degree (number of partnerships): 
bird.avg <- summarize(group_by(bird.dat, Color1), mean_L.n.partnerships=mean(log(n.partnerships+0.1)))
bird.dat <- merge(bird.dat, bird.avg)
bird.dat$rel_L.n.partnerships <- log(bird.dat$n.partnerships+0.1) - bird.dat$mean_L.n.partnerships # Relative degree
hist(bird.dat$rel_L.n.partnerships) # Relative degree is centered on 0

# Fit the model:

summary(mod <- glmer(cbind(prop.persisting*n.partnerships, (1 - prop.persisting)*n.partnerships) ~ studyyear + Status + scale(sample_eff) + scale(mean_L.n.partnerships) + scale(rel_L.n.partnerships) + (1|Color1), data=bird.dat, family='binomial'))


summary(mod.raw <- glmer(cbind(prop.persisting*n.partnerships, (1 - prop.persisting)*n.partnerships) ~ studyyear + Status + sample_eff + mean_L.n.partnerships + rel_L.n.partnerships + (1|Color1), data=bird.dat, family='binomial'))
visreg(mod.raw, 'mean_L.n.partnerships', scale='response')
visreg(mod.raw, 'rel_L.n.partnerships', scale='response')
# save(mod.raw, mod, bird.dat, file='manakin_data_individual_tradeoff_analysis.RData')

range(bird.dat$mean_L.n.partnerships)
range(bird.dat$rel_L.n.partnerships)
ndat1 <- data.frame(expand.grid(mean_L.n.partnerships=seq(0.09, 2.7, by=0.1), rel_L.n.partnerships=0))
ndat2 <- data.frame(expand.grid(mean_L.n.partnerships=1.7, rel_L.n.partnerships=seq(-2.1,1.8,by=0.1)))
head(ndat1); str(ndat1); head(ndat2); str(ndat2)
ndat1$studyyear <- '16-17'; ndat1$Status <- 'terr'; ndat1$sample_eff <- 74
ndat2$studyyear <- '16-17'; ndat2$Status <- 'terr'; ndat2$sample_eff <- 74

par(mfrow=c(2,2), bty='l', las=1)
plot(predict(mod.raw, newdata=ndat1, re.form=NA, type='response') ~ ndat1$mean_L.n.partnerships, pch=16, type='l', ylim=c(0,1), ylab='Predicted proportion persisting', xlab='Average degree')
ndat1.1 <- ndat1
ndat1.1$rel_L.n.partnerships <- -2.1
ndat1.2 <- ndat1
ndat1.2$rel_L.n.partnerships <- 1.8
polygon(x=c(ndat1.1$mean_L.n.partnerships, rev(ndat1.1$mean_L.n.partnerships)), y=c(predict(mod.raw, newdata=ndat1.1, re.form=NA, type='response'), rev(predict(mod.raw, newdata=ndat1.2, re.form=NA, type='response'))), border=NA, col=rgb(0,0,0,0.1))

plot(predict(mod.raw, newdata=ndat2, re.form=NA, type='response') ~ ndat2$rel_L.n.partnerships, pch=16, type='l', ylim=c(0,1), ylab='Predicted proportion persisting', xlab='Relative degree')
ndat2.1 <- ndat2
ndat2.1$mean_L.n.partnerships <- 0.09
ndat2.2 <- ndat2
ndat2.2$mean_L.n.partnerships <- 2.7
polygon(x=c(ndat2.1$rel_L.n.partnerships, rev(ndat2.1$rel_L.n.partnerships)), y=c(predict(mod.raw, newdata=ndat2.1, re.form=NA, type='response'), rev(predict(mod.raw, newdata=ndat2.2, re.form=NA, type='response'))), border=NA, col=rgb(0,0,0,0.1)) # now repeat this for a network simulation? unstandardized.





# Partial regression plots:
a <- visreg(mod, 'mean_L.n.partnerships', scale='response', plot=T, ylim=c(0,1))
b <- visreg(mod, 'rel_L.n.partnerships', scale='response', plot=T, ylim=c(0,1))

# Function to get standard error for samples with n > 3:
myse <- function(x){
	if(length(x)>3){
		return(sd(x)/sqrt(length(x)))
	} else {
		return(NA)
	}	
}

a2 <- aggregate(a[[2]][,c('visregRes','mean_L.n.partnerships')], by=list(ID=subset(bird.dat, !is.na(prop.persisting))$Color1), 'mean')
a3 <- aggregate(a[[2]][,c('visregRes','mean_L.n.partnerships')], by=list(ID=subset(bird.dat, !is.na(prop.persisting))$Color1), 'myse')
b[[2]]$ID <- subset(bird.dat, !is.na(prop.persisting))$Color1


dev.new(width=4.5, height=2) # Plot the among-individual positive association with degree, along with the within-individual trade-off:

par(mfrow=c(1,2), bty='l', las=1, mar=c(3,3,0.1,0.1), tck=-0.02, mgp=c(1.5,0.25,0), cex.lab=0.75, cex.axis=0.75)
plot(visregRes ~ mean_L.n.partnerships, a2, pch=16, cex=0.75, xlab='Among-individual log degree', ylab='Proportion maintained', ylim=c(0,1))
points(visregFit ~ mean_L.n.partnerships, a[[1]], type='l')
for(i in 1:nrow(a2)){
	if(!is.na(a3$visregRes)[i]){
		segments(x0=a2$mean_L.n.partnerships[i], y0=a2$visregRes[i] - a3$visregRes[i], y1=a2$visregRes[i] + a3$visregRes[i])
	}	
}

plot(visregRes ~ rel_L.n.partnerships, b[[2]], type='n', pch=16, cex=0.75, xlab='Within-individual relative log degree', ylab='Proportion maintained', ylim=c(0,1))
for(i in 1:nrow(a2)){
	tres <- subset(b[[2]], ID==a2$ID[i])
	if(nrow(tres)>3){
		mod <- lm(visregRes ~ rel_L.n.partnerships, data=tres)
		points(predict(mod, newdata=data.frame(rel_L.n.partnerships=sort(tres$rel_L.n.partnerships))) ~ sort(tres$rel_L.n.partnerships), type='l', col=rgb(0,0,0,0.75))
	}
}

summary(a2) # n = 565 bird-session observations; 152 individuals; 93 of which have >3 obs.



