
# Reciprocity and behavioral heterogeneity govern the stability of social networks 
# Roslyn Dakin and T. Brandt Ryder

# This script performs the main analysis of the manakin social networks, and performs a random rewiring test of stability
# Figure 1, 2

library(dplyr)
library(igraph)
library(lubridate)
library(lme4)
library(lmerTest)
library(visreg)
# Also requires: MuMIn package

# Get social interaction data:
year3 <- read.csv('filtered_interactions_nodups_times_durations_052418.csv', stringsAsFactors=F)[,c('date','nTime','Color1','Color2','Lek','studyyear','DL','sitefix')]; dim(year3)
year12 <- read.csv('filtered_interactions_nodups_times_durations_011018.csv', stringsAsFactors=F)[,c('date','nTime','Color1','Color2','Lek','studyyear','DL','sitefix')]; dim(year12)
ally <- rbind(year12, year3); head(ally); dim(ally) # Duplicate and reverse individual 1 & individual 2 to account for all interactions:
allyX <- ally
allyX$Color1 <- ally$Color2; allyX$Color2 <- ally$Color1
dim(ally); dim(allyX)
ally <- rbind(ally, allyX)
ally$Color1[ally$Color1=='OX-'] <- 'OX' # Correct ID error
ally$Color2[ally$Color2=='OX-'] <- 'OX'
ally$sitefix[ally$sitefix=='OX-'] <- 'OX'
all(levels(factor(ally$Color1))==levels(factor(ally$Color2))); dim(ally) # 73,770

# Get information on study logistics and sample hours:
stime1 <- read.csv('SampTimeXTerrxDate1516_2.csv', stringsAsFactors=F)[,-1]; dim(stime1); stime1$studyyear <- '15-16'
stime2 <- read.csv('SampTimeXTerrxDate1617.csv', stringsAsFactors=F)[,-1]; dim(stime2); stime2$studyyear <- '16-17'
stime3 <- read.csv('SampTimeXTerrxDate1718.csv', stringsAsFactors=F)[,-1]; dim(stime3); stime3$studyyear <- '17-18'
stime <- rbind(stime1, stime2, stime3); dim(stime)
stime$Date <- as.Date(stime$Date, format='%m/%d/%y')
stime$Terr[stime$Terr=='GXWA' | stime$Terr=='GXWB'] <- 'GXW' # Corrections
stime$Terr[stime$Terr=='MXOY'] <- 'MXYO'
levels(factor(stime$Terr))[!levels(factor(stime$Terr)) %in% levels(factor(ally$sitefix))]
levels(factor(ally$sitefix))[!levels(factor(ally$sitefix)) %in% levels(factor(stime$Terr))] # good
leklevels <- data.frame(old=levels(factor(stime$Lek)), new=levels(factor(ally$Lek)))
stime$Lek <- leklevels$new[match(stime$Lek, leklevels$old)]

manip <- read.csv('manipulation_info.csv', stringsAsFactors=F); dim(manip); head(manip)
manip$manipulation <- factor(manip$manipulation)
manip <- summarize(group_by(manip, studyyear, lek, DL), Tplus=table(manipulation)[4])

# Randomization data to use for stability filter (i.e., which partnershps are defined as significant):
fcheck <- read.csv('dyads_recording_session_plus_perm.csv', stringsAsFactors=F)
fcheck$filter3 <- (fcheck$edge.weight > fcheck$edge.weightPERM); head(fcheck); dim(fcheck) # 2933 session-edges

# Function to get stability:
stability_fun <- function(mat1, mat2, thresh=6, dl, yr){
	if( any(!rownames(mat1) %in% rownames(mat2)) | any(!rownames(mat2) %in% rownames(mat1)) ){
		newnames <- intersect(rownames(mat1), rownames(mat2))
		mat1 <- mat1[newnames, newnames]
		mat2 <- mat2[newnames, newnames]
	}
	mat1[mat1<thresh] <- 0; 	mat2[mat2<thresh] <- 0 # absolute occurrence threshold
	fc <- subset(fcheck, DL %in% dl & studyyear==yr & filter3==F) # sig. threshold (uses randomization)
	for(i in 1:nrow(mat1)){ 
		index <- sort(c(which(paste(colnames(mat1), rownames(mat1)[i], sep='_') %in% fc$dyad), which(paste(rownames(mat1)[i], colnames(mat1), sep='_') %in% fc$dyad)))
		mat1[i, index] <- 0
		index <- c(which(paste(colnames(mat2), rownames(mat2)[i], sep='_') %in% fc$dyad), which(paste(rownames(mat2)[i], colnames(mat2), sep='_') %in% fc$dyad))
		mat2[i, index] <- 0		
	}
	
	# Stability:	
	stability <- sum((mat2>0)[upper.tri(mat2)] & (mat1>0)[upper.tri(mat1)]) / sum(pmax((mat2>0)[upper.tri(mat2)], (mat1>0)[upper.tri(mat1)]))
	return(stability)	
}

# Quantify network properties:
net.prop <- data.frame(unique(ally[,c('studyyear','Lek','DL')]), yday=NA, nbird=NA, nint=NA, density=NA, modularity=NA, cluster.coef=NA, network.weight=NA, stability=NA, Tplus=0, hours.per=NA, cv_deg=NA)
rownames(net.prop) <- 1:nrow(net.prop)
dim(net.prop) # 86

options(warn=-1)
for(i in 1:nrow(net.prop)){
	yr <- net.prop$studyyear[i]
	lk <- net.prop$Lek[i]
	dl <- net.prop$DL[i]
	g1.0 <- subset(ally, studyyear==yr & Lek==lk & DL==dl)[,c('Color1','Color2','sitefix','date')] # Social interactions
	hours.per <- summarize(group_by(subset(stime, studyyear==yr & Lek==lk & DL==dl), Terr), hours=sum(sampTime))
	net.prop$hours.per[i] <- mean(hours.per$hours)
	net.prop$yday[i] <- yday(mean(as.Date(g1.0$date))) # Average day of the year
		
	g1a <- unique(g1.0[,c('Color1','Color2')])
	g1a <- subset(g1a, Color1<Color2) 
	temp <- summarize(group_by(g1.0, Color1, Color2), n=n())
	g1a$raw.weight <- temp$n[match(paste(g1a$Color1, g1a$Color2),  paste(temp$Color1, temp$Color2))]
	g1w <- graph.data.frame(data.frame(Color1=g1a$Color1, Color2=g1a$Color2, weight=g1a$raw.weight), directed=F) # Network
	
	net.prop$network.weight[i] <- mean(log(edge_attr(g1w)$weight))
	net.prop$nbird[i] <- length(names(V(g1w)))
	net.prop$nint[i] <- sum(edge_attr(g1w)$weight)
	net.prop$density[i] <- edge_density(g1w)
	net.prop$modularity[i] <- modularity(cluster_walktrap(g1w))
	net.prop$cluster.coef[i] <- transitivity(g1w, type='global')
	net.prop$cv_deg[i] <- sd(log(igraph::degree(g1w))) / mean(log(igraph::degree(g1w)))
		
	netmat <- (as.matrix(get.adjacency(g1w, type='both', attr='weight'))) # For stability	
	net.next <- subset(ally, studyyear==yr & Lek==lk & DL==dl+1)[,c('Color1','Color2','sitefix','date')]
	if(nrow(net.next)>0){
		net.next1a <- unique(net.next[,c('Color1','Color2')])
		net.next1a <- subset(net.next1a, Color1<Color2) 
		temp <- summarize(group_by(net.next, Color1, Color2), n=n())
		net.next1a$raw.weight <- temp$n[match(paste(net.next1a$Color1, net.next1a$Color2),  paste(temp$Color1, temp$Color2))]
		g1w.next <- graph.data.frame(data.frame(Color1=net.next1a$Color1, Color2=net.next1a$Color2, weight=net.next1a$raw.weight), directed=F) # Network
		netmat2 <- (as.matrix(get.adjacency(g1w.next, type='both', attr='weight')))
		net.prop$stability[i] <- stability_fun(netmat, netmat2, thresh=6, dl=dl, yr=yr)
	}
	mmm <- subset(manip, studyyear==yr & lek==lk & DL==dl) # Presence of manipulated individuals: 
	if(nrow(mmm)>0){
		net.prop$Tplus[i] <- mmm$Tplus
	}
	print(i)
}
options(warn=0)

net.prop$yday.adj <- ifelse(net.prop$yday>300, net.prop$yday-365, net.prop$yday) # Julian date, adjusted to be continuous within each field season
(interval.summary <- summarize(group_by(net.prop, studyyear, Lek), mean.span=mean(diff(sort(yday.adj)))))
summary(interval.summary$mean.span) # On averate, the recording sessions were 21 days apart (IQR 17-24 days)

# write.csv(net.prop, 'net.prop.csv')

# Candidate models of stability:stability
mod1 <- lmer(stability ~ studyyear + yday.adj + log(nbird) + network.weight + density + (1|Lek), data=net.prop)
mod2 <- lmer(stability ~ studyyear + yday.adj + log(nbird) + network.weight + modularity + (1|Lek), data=net.prop)
mod3 <- lmer(stability ~ studyyear + yday.adj + log(nbird) + network.weight + cluster.coef + (1|Lek), data=net.prop)
mod4 <- lmer(stability ~ studyyear + yday.adj + log(nbird) + network.weight + (1|Lek), data=net.prop)
mod5 <- lmer(stability ~ studyyear + yday.adj + log(nbird) + density + (1|Lek), data=net.prop)
mod6 <- lmer(stability ~ studyyear + yday.adj + log(nbird) + modularity + (1|Lek), data=net.prop)
mod7 <- lmer(stability ~ studyyear + yday.adj + log(nbird) + cluster.coef + (1|Lek), data=net.prop)
mod8 <- lmer(stability ~ studyyear + yday.adj + log(nbird) + (1|Lek), data=net.prop)

modtab <- AIC(mod1, mod2, mod3, mod4, mod5, mod6, mod7, mod8)
modtab[with(modtab, order(AIC)),]
MuMIn::model.sel(mod1, mod2, mod3, mod4, mod5, mod6, mod7, mod8, rank=AIC)

summary(mod1) # Best-fit model of stability

# Main conclusions are unchanged when accounting for sampling hours, experimental manipulations, or both, and when removing manipulated leks: 
mod1b <- lmer(stability ~ studyyear + yday.adj + hours.per + log(nbird) + network.weight + density + (1|Lek), data=net.prop); summary(mod1b) # p = 0.45
mod1c <- lmer(stability ~ studyyear + yday.adj + Tplus + log(nbird) + network.weight + density + (1|Lek), data=net.prop); summary(mod1c) # p = 0.43
mod1d <- lmer(stability ~ studyyear + yday.adj + hours.per + Tplus + log(nbird) + network.weight + density + (1|Lek), data=net.prop); summary(mod1d) # p > 0.45
mod1e <- lmer(stability ~ studyyear + yday.adj + hours.per + log(nbird) + network.weight + density + (1|Lek), data=subset(net.prop, Tplus==0)); summary(mod1e)

MuMIn::r.squaredGLMM(mod1)
MuMIn::r.squaredGLMM(mod3)
MuMIn::r.squaredGLMM(mod5)
MuMIn::r.squaredGLMM(mod8)
MuMIn::r.squaredGLMM(mod4)
MuMIn::r.squaredGLMM(mod7)
MuMIn::r.squaredGLMM(mod2)
MuMIn::r.squaredGLMM(mod6)

# Plot the best-fit model:
net.prop$nbirdL <- log(net.prop$nbird)
net.prop$studyyear <- factor(net.prop$studyyear)
mod <- lmer(stability ~ studyyear + yday.adj + nbirdL + network.weight + density + (1|Lek), data=subset(net.prop, !is.na(stability))); summary(mod)
b <- visreg(mod, 'network.weight', plot=F)
c <- visreg(mod, 'density', plot=F)

dev.new(width=4.5, height=2) # Partial residual:
par(mfrow=c(1,2), las=1, bty='l', mar=c(3,3,0.2,0.2), mgp=c(1.5,0.25,0), tck=-0.03)

plot(visregRes ~ network.weight, data=b$res, pch=16, col=rgb(0,0,0,1), cex=0.5, ylim=c(0,1), ylab='Network stability', xlab='Network weight', xlim=c(0.4,2.65))
points(visregFit ~ network.weight, data=b$fit, type='l', lwd=2.5)

plot(visregRes ~ density, data=c$res, pch=16, col=rgb(0,0,0,1), cex=0.5, ylim=c(0,1), ylab='', xlab='Network density', xlim=c(0.13,0.82))
points(visregFit ~ density, data=c$fit, type='l', lwd=2.5)

mod <- lmer(scale(stability)~ studyyear + scale(yday.adj) + scale(nbirdL) + scale(network.weight) + scale(density) + (1|Lek), data=subset(net.prop, !is.na(stability))); summary(mod) # Standardized for comparison
# save(mod, file='manakin_best_fit_regression_model.RData')

# Random rewiring of the observed social networks:
net.prop.rand <- net.prop; head(net.prop.rand); dim(net.prop.rand); net.prop.rand$stability <- NA

options(warn=-1)
for(i in 1:nrow(net.prop.rand)){	
	yr <- net.prop.rand$studyyear[i]
	lk <- net.prop.rand$Lek[i]
	dl <- net.prop.rand$DL[i]	
	g1.0 <- subset(ally, studyyear==yr & Lek==lk & DL==dl)[,c('Color1','Color2','sitefix','date')] 
	
	g1a <- unique(g1.0[,c('Color1','Color2')])
	g1a <- subset(g1a, Color1<Color2) 
	temp <- summarize(group_by(g1.0, Color1, Color2), n=n())
	g1a$raw.weight <- temp$n[match(paste(g1a$Color1, g1a$Color2),  paste(temp$Color1, temp$Color2))]
	g1w <- graph.data.frame(data.frame(Color1=g1a$Color1, Color2=g1a$Color2, weight=g1a$raw.weight), directed=F) # Network

	glist <- vector('list', length=100) # 100 permutations per network, stored in a list
	for(h in 1:100){
		set.seed(i)
		glist[[h]] <- rewire(g1w, each_edge(prob=1, loops=F, multiple=F)) # Rewires while holding network size, weight, density constant
	}

	netmat <- vector('list', length=100)
	for(h in 1:100){
		netmat[[h]] <- (as.matrix(get.adjacency(glist[[h]], type='both', attr='weight')))	
	}
	net.next <- subset(ally, studyyear==yr & Lek==lk & DL==dl+1)[,c('Color1','Color2','sitefix','date')]
	if(nrow(net.next)>0){
		net.next1a <- unique(net.next[,c('Color1','Color2')])
		net.next1a <- subset(net.next1a, Color1<Color2) 
		temp <- summarize(group_by(net.next, Color1, Color2), n=n())
		net.next1a$raw.weight <- temp$n[match(paste(net.next1a$Color1, net.next1a$Color2),  paste(temp$Color1, temp$Color2))]
		g1w.next <- graph.data.frame(data.frame(Color1=net.next1a$Color1, Color2=net.next1a$Color2, weight=net.next1a$raw.weight), directed=F) # Network
		netmat2 <- (as.matrix(get.adjacency(g1w.next, type='both', attr='weight')))
		store <- rep(NA, 100)	
		for(h in 1:100){
			netmat1 <- netmat[[h]]
			store[h] <- stability_fun(netmat1, netmat2, thresh=6, dl=dl, yr=yr)
		}
		net.prop.rand$stability[i] <- mean(store, na.rm=T)
	}
	print(i)
}
options(warn=0)


dev.new(width=2, height=1) # Histogram for Figure 1:
par(mar=c(2,2,0.1,0.1), mgp=c(2,0.25,0), tck=-0.02, cex.lab=0.5)
plot(density(net.prop$stability, na.rm=T), xlim=c(0,1), main='', bty='n', las=1, xlab='', ylab='', xaxt='n', yaxt='n')
axis(1, at=c(0,0.5,1))
abline(v=mean(net.prop.rand$stability, na.rm=T))
polygon(x=c(t.test(net.prop.rand$stability)$conf.int[1], t.test(net.prop.rand$stability)$conf.int[2], t.test(net.prop.rand$stability)$conf.int[2],t.test(net.prop.rand$stability)$conf.int[1]), y=c(0,0,2,2), col=rgb(0,0,0,0.1), border=NA)
abline(v=mean(net.prop$stability, na.rm=T), lty=3)
polygon(x=c(density(net.prop$stability, na.rm=T)$x, rev(density(net.prop$stability, na.rm=T)$x)), y=c(density(net.prop$stability, na.rm=T)$y, rep(0, length(density(net.prop$stability, na.rm=T)$y))), col=rgb(1,0,0,0.1), border=NA)



