
# Reciprocity and behavioral heterogeneity govern the stability of social networks 
# Roslyn Dakin and T. Brandt Ryder

# This script provides descriptive statistics and visualizatoins of example social networks
# Figure 1

library(igraph)
library(dplyr)
library(lme4)

# Network properties:
net.prop <- read.csv('net.prop.csv', stringsAsFactors=F); head(net.prop); dim(net.prop)

# Get social interaction data:
year3 <- read.csv('filtered_interactions_nodups_times_durations_052418.csv', stringsAsFactors=F)[,c('date','nTime','Color1','Color2','Lek','studyyear','DL','sitefix')]; dim(year3)
year12 <- read.csv('filtered_interactions_nodups_times_durations_011018.csv', stringsAsFactors=F)[,c('date','nTime','Color1','Color2','Lek','studyyear','DL','sitefix')]; dim(year12)
ally <- rbind(year12, year3); head(ally); dim(ally) # Duplicate and reverse individual 1 & individual 2 to account for all interactions:
allyX <- ally
allyX$Color1 <- ally$Color2; allyX$Color2 <- ally$Color1
dim(ally); dim(allyX)
ally <- rbind(ally, allyX)
ally$Color1[ally$Color1=='OX-'] <- 'OX' # Correct ID error
ally$Color2[ally$Color2=='OX-'] <- 'OX'
ally$sitefix[ally$sitefix=='OX-'] <- 'OX'
all(levels(factor(ally$Color1))==levels(factor(ally$Color2))); dim(ally) # 73,770


# Plot histograms of network properties and example networks in a gradient to illustrate, for figure 1:

colorfun <- colorRampPalette(c("red", "blue"))
colorset <- colorfun(101); colorset_transp <- paste(colorset, '75', sep='')

dev.new(width=4.5, height=2) # 
par(mfrow=c(1,2), las=1, bty='l', mar=c(3,3,0.2,0.2), mgp=c(1.5,0.25,0), tck=-0.03)
plot(density ~ log(nbird), pch=16, data=net.prop, xlab='Network size (# individuals)', ylab='Density', cex=0.75, cex.axis=0.75, cex.lab=0.75, xaxt='n')
axis(1, at=log(c(2,5,10,20,40)), labels=c(2,5,10,20,40), cex.axis=0.75)
plot(network.weight ~ log(nbird), pch=16, data=net.prop, xlab='', ylab='Network strength', cex=0.75, cex.axis=0.75, cex.lab=0.75, xaxt='n')
axis(1, at=log(c(2,5,10,20,40)), labels=c(2,5,10,20,40), cex.axis=0.75)

colorfun <- colorRampPalette(c("blue", "red"))
colorset <- colorfun(101); colorset_transp <- paste(colorset, '99', sep='')

dev.new(width=6.75, height=3) # 
layout(matrix(c(rep(rep(1:3, each=5), 2), 4:18), nrow=3, ncol=15, byrow=T))
par(las=1, bty='l', mar=c(3,3,0.2,0.2), mgp=c(1.5,0.25,0), tck=-0.03)

(h <- hist(log(net.prop$nbird), plot=F, breaks=9))
(d <- density(log(net.prop$nbird))); d$y <- (d$y/max(d$y))*max(h$counts)
d <- subset(data.frame(y=d$y, x=d$x), x > 0.5 & x < 4)
plot(h, main='', xlim=c(0.5,4), ylim=c(0,30), cex.axis=0.75, cex.lab=0.75, col=rgb(0,0,0,0.1), border=NA, xlab='Size (# individuals)', xaxt='n'); axis(1, at=log(c(2,5,10,20,40)), labels=c(2,5,10,20,40), cex.axis=0.75)
polygon(x=c(d$x, rev(d$x)), y=c(rep(0,length(d$y)), rev(d$y)), col=rgb(0,0,1,0.15), border=NA)

(h <- hist(net.prop$density, plot=F))
(d <- density(net.prop$density)); d$y <- (d$y/max(d$y))*max(h$counts)
d <- subset(data.frame(y=d$y, x=d$x), x > 0 & x < 1)
plot(h, main='', xlim=c(0,1), ylim=c(0,30), cex.axis=0.75, cex.lab=0.75, col=rgb(0,0,0,0.1), border=NA, xlab='Density')
polygon(x=c(d$x, rev(d$x)), y=c(rep(0,length(d$y)), rev(d$y)), col=rgb(0,0,1,0.15), border=NA)

(h <- hist(net.prop$network.weight, plot=F))
(d <- density(net.prop$network.weight)); d$y <- (d$y/max(d$y))*max(h$counts)
d <- subset(data.frame(y=d$y, x=d$x), x > 0 & x < 3)
plot(h, main='', xlim=c(0,3), ylim=c(0,30), cex.axis=0.75, cex.lab=0.75, col=rgb(0,0,0,0.1), border=NA, xlab='Strength', xaxt='n'); axis(1, at=c(0:3), labels=0:3, cex.axis=0.75)
polygon(x=c(d$x, rev(d$x)), y=c(rep(0,length(d$y)), rev(d$y)), col=rgb(0,0,1,0.15), border=NA)

par(mar=rep(0,4))
for(j in c(52,86,43,7,37)){
	yr <- net.prop$studyyear[j]
	lk <- net.prop$Lek[j]
	dl <- net.prop$DL[j]
	g1.0 <- subset(ally, studyyear==yr & Lek==lk & DL==dl)[,c('Color1','Color2','sitefix','date')]
		
	g1a <- unique(g1.0[,c('Color1','Color2')])
	g1a <- subset(g1a, Color1<Color2) 
	temp <- summarize(group_by(g1.0, Color1, Color2), n=n())
	g1a$raw.weight <- temp$n[match(paste(g1a$Color1, g1a$Color2),  paste(temp$Color1, temp$Color2))]
	g1 <- graph.data.frame(data.frame(Color1=g1a$Color1, Color2=g1a$Color2, weight=g1a$raw.weight), directed=F) # Network

	gcol <- log(net.prop$nbird[j])/log(54)
	plot(g1, layout=layout_in_circle, vertex.size=20, vertex.color=colorset_transp[round(gcol*100 + 1)], vertex.frame.color=colorset_transp[round(gcol*100 + 1)], layout=layout_in_circle, edge.width=0.75, edge.color=colorset_transp[round(gcol*100 + 1)], vertex.label=NA)
}
for(j in c(13,74,23,54,26)){
	yr <- net.prop$studyyear[j]
	lk <- net.prop$Lek[j]
	dl <- net.prop$DL[j]
	g1.0 <- subset(ally, studyyear==yr & Lek==lk & DL==dl)[,c('Color1','Color2','sitefix','date')]
	g1a <- unique(g1.0[,c('Color1','Color2')])
	g1a <- subset(g1a, Color1<Color2) 
	temp <- summarize(group_by(g1.0, Color1, Color2), n=n())
	g1a$raw.weight <- temp$n[match(paste(g1a$Color1, g1a$Color2),  paste(temp$Color1, temp$Color2))]
	g1 <- graph.data.frame(data.frame(Color1=g1a$Color1, Color2=g1a$Color2, weight=g1a$raw.weight), directed=F) # Network
	gcol <- net.prop$density[j]
	plot(g1, layout=layout_in_circle, vertex.size=20, vertex.color=colorset_transp[round(gcol*100 + 1)], vertex.frame.color=colorset_transp[round(gcol*100 + 1)], layout=layout_in_circle, edge.width=0.75, edge.color=colorset_transp[round(gcol*100 + 1)], vertex.label=NA)
}
for(j in c(84,24,60,49,29)){
	yr <- net.prop$studyyear[j]
	lk <- net.prop$Lek[j]
	dl <- net.prop$DL[j]
	g1.0 <- subset(ally, studyyear==yr & Lek==lk & DL==dl)[,c('Color1','Color2','sitefix','date')]
	g1a <- unique(g1.0[,c('Color1','Color2')])
	g1a <- subset(g1a, Color1<Color2) 
	temp <- summarize(group_by(g1.0, Color1, Color2), n=n())
	g1a$raw.weight <- temp$n[match(paste(g1a$Color1, g1a$Color2),  paste(temp$Color1, temp$Color2))]
	g1 <- graph.data.frame(data.frame(Color1=g1a$Color1, Color2=g1a$Color2, weight=g1a$raw.weight), directed=F) # Network
	gcol <- net.prop$network.weight[j]/3
	plot(g1, layout=layout_in_circle, vertex.size=20, vertex.color=colorset_transp[round(gcol*100 + 1)], vertex.frame.color=colorset_transp[round(gcol*100 + 1)], layout=layout_in_circle, edge.width=exp(net.prop$network.weight[j])/3.4, edge.color=colorset_transp[round(gcol*100 + 1)], vertex.label=NA)
}

# Sample networks for figure 2:
colorset <- colorfun(101); colorset_transp <- paste(colorset, '99', sep='')
dev.new(width=2, height=0.5)
par(mfrow=c(1,4), mar=rep(0,4))
for(j in c(43,45,59,68)){ # 8,34,7,73
	yr <- net.prop$studyyear[j]
	lk <- net.prop$Lek[j]
	dl <- net.prop$DL[j]
	g1.0 <- subset(ally, studyyear==yr & Lek==lk & DL==dl)[,c('Color1','Color2','sitefix','date')]
	g1a <- unique(g1.0[,c('Color1','Color2')])
	g1a <- subset(g1a, Color1<Color2) 
	temp <- summarize(group_by(g1.0, Color1, Color2), n=n())
	g1a$raw.weight <- temp$n[match(paste(g1a$Color1, g1a$Color2),  paste(temp$Color1, temp$Color2))]
	g1 <- graph.data.frame(data.frame(Color1=g1a$Color1, Color2=g1a$Color2, weight=g1a$raw.weight), directed=F) # Network
	gcol <- log(net.prop$nbird[j])/log(54)
	plot(g1, layout=layout_with_lgl, vertex.size=(igraph::degree(g1)+2)*4, vertex.color=colorset_transp[round(gcol*100 + 1)], vertex.frame.color=colorset_transp[round(gcol*100 + 1)], layout=layout_in_circle, edge.width=log(edge_attr(g1)$weight+1), edge.color=colorset_transp[round(gcol*100 + 1)], vertex.label=NA)
}



# Plot repeatabilities for size, density, strength, stability (proportion of total variance due to differences among leks):
myrep <- function(mod){
	data.frame(VarCorr(mod))[,'vcov'][1]/sum(data.frame(VarCorr(mod))[,'vcov'])
}
(repeats <- data.frame(y=c('log(nbird)','density','network.weight','stability'), rep=NA, lower=NA, upper=NA))
for(i in 1:4){
	mod <- lmer(as.formula(paste(repeats$y[i], '~ studyyear + yday.adj + (1|Lek)')), data=net.prop)
	repeats$rep[i] <- myrep(mod)
	set.seed(i)
	brep <- bootMer(mod, FUN='myrep', nsim=5000)$t # Slow w/ 5000 iter.
	repeats[i,c('lower','upper')] <- quantile(brep, c(0.025, 0.975))
	print(i)
}
dev.new(width=2.25, height=2) # 
par(las=1, bty='l', mar=c(3,3,0.2,0.2), mgp=c(1.5,0.25,0), tck=-0.03, cex.axis=0.75, cex.lab=0.75)
plot(repeats$rep ~ c(1:4), pch=16, ylab='Repeatability', xlab='', xaxt='n', xlim=c(0.5,4.5), ylim=c(-0.05,1), cex=0.75)
axis(1, at=1:4, labels=c('Size','Density','Weight','Stability'), cex.axis=0.25)
segments(x0=1:4, y0=repeats$lower, y1=repeats$upper)
abline(h=0, lty=3)


# Plot stability of example networks with a range of stabilities:

dev.new(width=5, height=5)
par(mar=rep(0,4), mfrow=c(2,2))

i <- 72

(yr <- net.prop$studyyear[i])
(lk <- net.prop$Lek[i])
(dl <- net.prop$DL[i])
stab <- net.prop$stability[i]
g1.0 <- subset(ally, studyyear==yr & Lek==lk & DL==dl)[,c('Color1','Color2','sitefix','date')]
g1a <- unique(g1.0[,c('Color1','Color2')])
g1a <- subset(g1a, Color1<Color2) 
temp <- summarize(group_by(g1.0, Color1, Color2), n=n())
g1a$raw.weight <- temp$n[match(paste(g1a$Color1, g1a$Color2),  paste(temp$Color1, temp$Color2))]
g1 <- graph.data.frame(data.frame(Color1=g1a$Color1, Color2=g1a$Color2, weight=g1a$raw.weight), directed=F) # Network

step <- expand.grid(names(V(g1)), names(V(g1)))
step <- subset(step, Var1!=Var2)
step <- subset(step, as.character(Var1)<as.character(Var2))
step$weight <- 0; step$w2 <- 0; step$change <- NA
for(h in 1:nrow(step)){
	step$weight[h] <- nrow(subset(g1.0, (Color1==step$Var1[h] & Color2==step$Var2[h])))
}

step$color <- rgb(0,0,0,0)
colorfun <- colorRampPalette(c("red", "blue"))
colorset <- colorfun(101); colorset_transp <- paste(colorset, 75, sep='')
step$color[step$weight>0] <- colorset_transp[round(stab*100 + 1)]

g2.0 <- subset(ally, studyyear==yr & Lek==lk & DL==dl+1)[,c('Color1','Color2','sitefix','date')]
g2a <- unique(g2.0[,c('Color1','Color2')])
g2a <- subset(g2a, Color1<Color2) 
temp <- summarize(group_by(g2.0, Color1, Color2), n=n())
g2a$raw.weight <- temp$n[match(paste(g2a$Color1, g2a$Color2),  paste(temp$Color1, temp$Color2))]
g2 <- graph.data.frame(data.frame(Color1=g2a$Color1, Color2=g2a$Color2, weight=g2a$raw.weight), directed=F) # Network


(g2add <- names(V(g1))[! names(V(g1)) %in% names(V(g2))])
(g1add <- names(V(g2))[! names(V(g2)) %in% names(V(g1))])

g1 <- add_vertices(g1, nv=length(g1add))
g2 <- add_vertices(g2, nv=length(g2add))

g1 <- set_vertex_attr(g1, 'name', value=ifelse(is.na(names(V(g1))), g1add, names(V(g1))))
g2 <- set_vertex_attr(g2, 'name', value=ifelse(is.na(names(V(g2))), g2add, names(V(g2))))

layout_with_fr(g1)

temp.lay <- layout_in_circle(g1)
plot(g1, layout=temp.lay, vertex.size=15, vertex.color=colorset_transp[round(stab*100 + 1)], vertex.frame.color=colorset_transp[round(stab*100 + 1)], edge.width=log(edge_attr(g1)$weight+1), edge.color=colorset_transp[round(stab*100 + 1)], vertex.label=NA)
plot(g2, layout=temp.lay[match(names(V(g2)), names(V(g1))),], vertex.size=15, vertex.color=colorset_transp[round(stab*100 + 1)], vertex.frame.color=colorset_transp[round(stab*100 + 1)], edge.width=log(edge_attr(g1)$weight+1), edge.color=colorset_transp[round(stab*100 + 1)], vertex.label=NA)
text(0,0,round(stab,2))

i <- 14

(yr <- net.prop$studyyear[i])
(lk <- net.prop$Lek[i])
(dl <- net.prop$DL[i])
stab <- net.prop$stability[i]
g1.0 <- subset(ally, studyyear==yr & Lek==lk & DL==dl)[,c('Color1','Color2','sitefix','date')]
g1a <- unique(g1.0[,c('Color1','Color2')])
g1a <- subset(g1a, Color1<Color2) 
temp <- summarize(group_by(g1.0, Color1, Color2), n=n())
g1a$raw.weight <- temp$n[match(paste(g1a$Color1, g1a$Color2),  paste(temp$Color1, temp$Color2))]
g1 <- graph.data.frame(data.frame(Color1=g1a$Color1, Color2=g1a$Color2, weight=g1a$raw.weight), directed=F) # Network

step <- expand.grid(names(V(g1)), names(V(g1)))
step <- subset(step, Var1!=Var2)
step <- subset(step, as.character(Var1)<as.character(Var2))
step$weight <- 0; step$w2 <- 0; step$change <- NA
for(h in 1:nrow(step)){
	step$weight[h] <- nrow(subset(g1.0, (Color1==step$Var1[h] & Color2==step$Var2[h])))
}

step$color <- rgb(0,0,0,0)
colorfun <- colorRampPalette(c("red", "blue"))
colorset <- colorfun(101); colorset_transp <- paste(colorset, 75, sep='')
step$color[step$weight>0] <- colorset_transp[round(stab*100 + 1)]

g2.0 <- subset(ally, studyyear==yr & Lek==lk & DL==dl+1)[,c('Color1','Color2','sitefix','date')]
g2a <- unique(g2.0[,c('Color1','Color2')])
g2a <- subset(g2a, Color1<Color2) 
temp <- summarize(group_by(g2.0, Color1, Color2), n=n())
g2a$raw.weight <- temp$n[match(paste(g2a$Color1, g2a$Color2),  paste(temp$Color1, temp$Color2))]
g2 <- graph.data.frame(data.frame(Color1=g2a$Color1, Color2=g2a$Color2, weight=g2a$raw.weight), directed=F) # Network


(g2add <- names(V(g1))[! names(V(g1)) %in% names(V(g2))])
(g1add <- names(V(g2))[! names(V(g2)) %in% names(V(g1))])

g1 <- add_vertices(g1, nv=length(g1add))
g2 <- add_vertices(g2, nv=length(g2add))

g1 <- set_vertex_attr(g1, 'name', value=ifelse(is.na(names(V(g1))), g1add, names(V(g1))))
g2 <- set_vertex_attr(g2, 'name', value=ifelse(is.na(names(V(g2))), g2add, names(V(g2))))

layout_with_fr(g1)

temp.lay <- layout_in_circle(g1)
plot(g1, layout=temp.lay, vertex.size=15, vertex.color=colorset_transp[round(stab*100 + 1)], vertex.frame.color=colorset_transp[round(stab*100 + 1)], edge.width=log(edge_attr(g1)$weight+1), edge.color=colorset_transp[round(stab*100 + 1)], vertex.label=NA)
plot(g2, layout=temp.lay[match(names(V(g2)), names(V(g1))),], vertex.size=15, vertex.color=colorset_transp[round(stab*100 + 1)], vertex.frame.color=colorset_transp[round(stab*100 + 1)], edge.width=log(edge_attr(g1)$weight+1), edge.color=colorset_transp[round(stab*100 + 1)], vertex.label=NA)
text(0,0,round(stab,2))


